--- Internal sync interface module via file
-- @author Oskar Eisemuth
-- @copyright 2021-2023
-- @module commonapi._shareddatafile
local syncstorage = {}

require "serialize"

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi2._shareddatafile")

--  commonapi._reloadModule("_shareddatafile"); print(commonapi._shareddatafile.getSerial("repository_street"))
function syncstorage.setResetOnGameLoad(key, b)
	return (b == true)	-- we always reset on game load with this backend
end

function syncstorage.getSerial(key)
	local fullfilename = commonapi._info.path.."syncobject-"..key
	local syncfile, err = io.open(fullfilename, "r")
	if (syncfile == nil) then
		_log(err)
		return nil;
	end
	local line = syncfile:read("*l")
	syncfile:close()
	if (line ~= nil) then
		local serial = tonumber(line:sub(3))
		return serial
	end
	return nil
end

function syncstorage.get(key)
	local fullfilename = commonapi._info.path.."syncobject-"..key
	local ok
	local result

	local env = {
		data = function()
			return nil
		end
	}
	local luafn, err = loadfile(fullfilename, "bt", env)
	if (luafn == nil) then
		_log(err)
		return nil
	end

	ok, err = pcall(luafn)
	if (ok == false) then
		_log(err)
		return nil
	end

	ok, result = pcall(env.data)
	if (ok == false) then
		_log(result)
		return nil
	end
	return result
end


function syncstorage.set(key, o)
	--_log("syncstorage.set")
	--print(debug.traceback())
	local fullfilename = commonapi._info.path.."syncobject-"..key
	local oldserial = syncstorage.getSerial(key)

	if (oldserial == nil) then
		oldserial = 0
	end
	local newserial = oldserial + 1;
	if (newserial > 1000) then
		newserial = 1
	end

	--	local serializeData = "-- "..newserial.."\n\n"..serializeStr(o)
	local t = _utils.serializeDataToFn(o)
	local serializeData = "-- "..newserial.."\n\n"..t

	local syncfile = io.open(fullfilename, "w")
	if (syncfile) then
		syncfile:write(serializeData)
		syncfile:close()
	else
		_log("Can't write to ", fullfilename)
	end
	return newserial
end

return syncstorage