local cs = {}
cs.testmode = false
cs._doc = {}
local cs_members = {}

local _utils = commonapi.utils
local hs = commonapi.rerequire("commonapi2.lineextradetails.helperscript")

--[[
conobj = commonapi.rerequire("commonapi2.lineextradetails.constructionscript")
x = conobj.new(22913)
conobj.new(136100):setParameter("lmp_t174_Farbe_FZ", 3):doIt()
conobj.new(22913):setParameter("lmp_t174_Farbe_FZ", 1):doIt()

--]]


local function cs_members_create()
	if (#cs_members > 0) then
		return
	end
	for k, v in pairs(cs._doc) do
		cs_members[#cs_members + 1] = v.name
	end

	if (cs.__doc__ ~= nil) then
		for k, v in pairs(cs._doc) do
			if (v.calling) then
				cs.__doc__[v.name] = v.calling.."   "..v.desc
			else
				cs.__doc__[v.name] = v.desc
			end
		end
	end
end
--cs.__members = cs_members



function cs.newEnv(testmode)
	local o = {}
	local mt = {
		__index = cs,
	}
	setmetatable(o, mt)
	o.testmode = testmode
	o.new = function(nentityId)
		return cs._new(o, nentityId)
	end
	return o
end


cs._doc[#cs._doc+1] = {
	name = "new",
	calling = "(int entityId) -> ConstructionScriptObject",
	params = {
		{ "entityId", "integer", "entityId or -1" },
	},
	ret = "new ConstructionObject",
	desc = "Creates a construction script object, pass -1 for new construnction.\nNOTE: In line events you can use conobj.new(<yourentityid>) to access constructions"
}
function cs.new(entityId)
	return cs._new(cs, entityId)
end

function cs._new(parent, entityId)
	local o = {}
	local mt = {
		__index = parent,
	}
	setmetatable(o, mt)
	o.new = function(nentityId)
		return cs._new(parent, nentityId)
	end
	o.testmode = (parent.testmode == true)
	if (entityId == nil) then
		entityId = -1
	end
	o.entityId = entityId

	local con
	if (api.engine and entityId ~= -1) then
		con = api.engine.getComponent(entityId, api.type.ComponentType.CONSTRUCTION)
	end
	if (con ~= nil) then
		o.oldcon = con
		o._params = hs.deepcopy(con.params)
		o._filename = con.fileName
		o._transf = hs.deepcopy(con.transf)

		local playerComponent = api.engine.getComponent(entityId, api.type.ComponentType.PLAYER_OWNED)
		if (playerComponent ~= nil) then
			o._playerEntity = playerComponent.player
		else
			o._playerEntity = -1
		end

		local nameComponent = api.engine.getComponent(entityId, api.type.ComponentType.NAME)
		if (nameComponent ~= nil) then
			o._name = nameComponent.name
		else
			o._name = ""
		end
	else
		if (entityId ~= -1) then
			return nil
		end
		o._params = {}
		o._filename = ""
		o._name = ""
		o._transf = api.type.Mat4f.new()
		o._playerEntity = api.engine.util.getPlayer()
	end
	return o
end


cs._doc[#cs._doc+1] = {
	name = "setParameter",
	calling = "(String name, value) -> self",
	params = {
		{ "self" },
		{ "name", "string", "parameter name" },
		{ "value", "", "" }
	},
	ret = "self",
	desc = "Set a parameter to value"
}
function cs:setParameter(name, value)
	self._params[name] = value
	return self
end

cs._doc[#cs._doc+1] = {
	name = "getParameters",
	params = {
		{ "self" }
	},
	ret = "table",
	desc = "Returns parameter table"
}
function cs:getParameters(name, value)
	return self._params
end


cs._doc[#cs._doc+1] = {
	name = "setTransf",
	params = {
		{ "self" },
		{ "transf", "Mat4f", "Position" },
	},
	ret = "self",
	desc = "Set position matrix"
}
function cs:setTransf(transf)
	self._transf = hs.deepcopy(transf)
	return self._transf
end

cs._doc[#cs._doc+1] = {
	name = "setFilename",
	params = {
		{ "self" },
		{ "fileName", "string", "Construction filename" },
	},
	ret = "self",
	desc = "Set the construction filename"
}
function cs:setFilename(fileName)
	self._filename = fileName
	return self._filename
end

cs._doc[#cs._doc+1] = {
	name = "setPlayerEntity",
	calling = "(int playerEntityId) -> table",
	params = {
		{ "self" },
		{ "playerEntityId", "int", "" },
	},
	ret = "self",
	desc = "Set the player entity"
}
function cs:setPlayerEntity(playerEntityId)
	self._playerEntity = playerEntityId
	return self
end

cs._doc[#cs._doc+1] = {
	name = "doIt",
	params = {
		{ "self" }
	},
	desc = "Apply the change"
}
function cs:doIt()
	local proposal = api.type.SimpleProposal.new()

	local connew = api.type.SimpleProposal.ConstructionEntity.new()
	connew.fileName = self._filename
	connew.params = self._params
	connew.transf = self._transf
	connew.playerEntity = self._playerEntity
	connew.name = self._name
	if (self.entityId ~= -1) then
		proposal.constructionsToAdd[1] = connew
		proposal.constructionsToRemove = { self.entityId }
		proposal.old2new[self.entityId] = 0
	else
		proposal.constructionsToAdd[1] = connew
	end
	local ctx = api.type.Context:new()
	ctx.checkTerrainAlignment = true
	ctx.cleanupStreetGraph = true
	ctx.gatherBuildings = true
	ctx.gatherFields = true

	local cmd = api.cmd.make.buildProposal(proposal, ctx, true)

	if (self.testmode ~= true) then
		api.cmd.sendCommand(cmd,  function(cmd, success)
			if (success ~= true) then
				commonapi.dmp(cmd)
			end
		end)
	end
end

--cs_members_create()
hs.createDoc(cs)
return cs