local hs = {}

local vecstart = "std::vector"
local vecstartlen = #vecstart

local deepMapping = {
	CVec4f = "Vec4f",
	CVec3f = "Vec3f",
	CVec2f = "Vec2f",
}
local function deepcopy2(o)
	local t = type(o)
	if ( t == "table" ) then
		local newtable = {}
		for k, v in pairs(o) do
			newtable[k] = deepcopy2(v)
		end
		return newtable
	elseif ( t == "userdata" ) then
		local metatable = getmetatable(o)
		if (metatable.__name ~= nil) then
			local typename = string.gsub(metatable.__name, "^sol%.", "")
			typename = string.gsub(typename, "%*$", "")

			if (deepMapping[typename] ~= nil) then
				typename = deepMapping[typename]
				if (type(typename) ~= "string") then
					return typename(o)
				end
			end
			if (api.type[typename] ~= nil) then
				local newobj = api.type[typename].new()
				if(metatable.__members ~= nil) then
					local members = metatable.__members
					for i = 1, #members do
						local key = members[i]
						if (key ~= "new") then
							local v = o[key]
							newobj[key] = deepcopy2(v)
						end
					end
				else
					for k, v in pairs(o) do
						newobj[k] = deepcopy2(v)
					end
				end
				return newobj
			else
				--print(string.sub(typename, 1, vecstartlen))
				if (string.sub(typename, 1, vecstartlen) == vecstart) then
					local newtable = {}
					for k, v in pairs(o) do
						newtable[k] = deepcopy2(v)
					end
					return newtable
				end
			end
			print("Don't know how to deepcopy: ", typename)
			return nil
		end
		print("Error at deepcopy, no metatable for usertype")
	end
	return o
end

deepMapping.CMat4f = function(mat)
	local v0 = mat:cols(0)
	local v1 = mat:cols(1)
	local v2 = mat:cols(2)
	local v3 = mat:cols(3)
	return api.type.Mat4f.new(v0, v1, v2, v3)
end

deepMapping["lua::Table"] = function(luaTable)
	local t = {}
	for k, v in pairs(luaTable) do
		t[k] = deepcopy2(v)
	end
	return t
end

hs.deepcopy = deepcopy2

hs.createDoc = function(o)
	if (commonapi._native ~= nil and commonapi._native.utils and commonapi._native.utils.createUnorderedMapString) then
		local doc = commonapi._native.utils.createUnorderedMapString()
		--cs.__doc__ = {}
		for k, v in pairs(o._doc) do
			if (type(o[v.name]) == "function") then
				local calling = {}
				if (v.instance == true) then
					calling[#calling+1] = "self"
				end
				if (v.params ~= nil) then
					for paramid, paramdata in ipairs(v.params) do
						if (paramdata[2] ~= nil) then
							calling[#calling+1] = (paramdata[2] or "").." "..paramdata[1]
						else
							calling[#calling+1] = paramdata[1]
						end
					end
				end
				local desc = "("..table.concat(calling,", ")..")"
				if (v.ret) then
					desc = desc .. " -> "..v.ret
				end
				if (v.desc) then
					desc = desc .. "   -- "..v.desc
				end
				doc[v.name] = desc
			else
				doc[v.name] = v.desc
			end
		end
		o.__doc__ = doc
	end
end

return hs