local lineextradetails = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi2.lineextradetails")
local _syncdata = commonapi._syncdata

local vs
local vsok, vsobj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.vehiclescript")
if (vsok == true and type(vsobj) == "table") then
	vs = vsobj
else
	print(vsobj)
end


local linesextradetails_sync =
	_syncdata.SyncObject:new(
	{
		key = "linesextradetails",
		data = {}
	}
)

local datastorage = linesextradetails_sync:getDataOrInit({})


function lineextradetails.applySavegameData(data)
	if (type(data) ~= "table") then
		return
	end
	-- TODO: Better Version check
	if (data.version == 1) then
		linesextradetails_sync:syncData(data)
		datastorage = linesextradetails_sync:getData()
		if (commonapi._native ~= nil and commonapi._native.LabelSystem ~= nil) then
			commonapi._native.LabelSystem.updateData(datastorage)
		end
		-- TODO: Verifiy Data?
	end
end

function lineextradetails.notifyDataChange()
	_log("notifyDataChange")
	datastorage = linesextradetails_sync:getData()
end


function lineextradetails.getStateData()
	return linesextradetails_sync:getData()
end

function lineextradetails.syncData()
	_log("syncData")
	lineextradetails.verifyLines()

	linesextradetails_sync:syncData()
	datastorage = linesextradetails_sync:getData()
	if (commonapi._native.LabelSystem ~= nil) then
		commonapi._native.LabelSystem.updateData(datastorage)
	end
	commonapi_gamescript.saveData()
end


function lineextradetails.getOrCreateLineData(lineId)
	if (datastorage.version ~= 1) then
		datastorage.version = 1

	end

	if (datastorage.lines == nil) then
		datastorage.lines = {}
	end

	if (datastorage.lines[lineId] == nil) then
		datastorage.lines[lineId] = {}
	end
	local linedetails = datastorage.lines[lineId]
	return linedetails
end

function lineextradetails.getLineData(lineId)
	if (datastorage == nil) then
		return nil
	end
	if (datastorage.lines == nil) then
		return nil
	end
	if (datastorage.lines[lineId] == nil) then
		return nil
	end
	local linedetails = datastorage.lines[lineId]
	return linedetails
end


function lineextradetails.runScriptAtEvent(eventdata, name, trace)
	if (eventdata == nil) then
		return
	end

	if (eventdata.id == nil) then
		return
	end

	if (eventdata.line == nil or eventdata.line < 1) then
		return
	end

	if (eventdata.stopIndex < 0) then
		return
	end

	if (trace) then
		_log(name.." runScriptAtEvent vehicle ", eventdata.id,  eventdata.line, eventdata.stopIndex )
	end
	local linedetails = lineextradetails.getLineData(eventdata.line)
	if (linedetails == nil) then
		if (trace) then
			_log(name.." linedetails not set", eventdata.id)
		end
		return
	end

	local data = linedetails[name]
	if (data == nil) then
		if (trace) then
			_log(name.." not set", eventdata.id)
		end
		return
	end

	local uniqueIdList = lineextradetails.getUniqueIdsForLineStops(eventdata.line)
	local uniqueStopId = uniqueIdList[eventdata.stopIndex+1]
	if (uniqueStopId == nil) then
		return
	end
	if (data[uniqueStopId] ~= nil) then
		local stopData = data[uniqueStopId]
		if (stopData.runScriptSrc ~= nil and stopData.runScript == true) then
			if (trace) then
				_log("linedetails."..name.." runScriptSrc", eventdata.id)
			end
			if (vs ~= nil) then
				local scriptsrcid = "commonapi-"..name..".lineId-"..eventdata.line..".stopId-"..uniqueStopId..".vehicleId-"..eventdata.id
				local ok, err = vs.run(eventdata.id, stopData.runScriptSrc, scriptsrcid)
				if (ok ~= true) then
					print(err)
				end
			end
		end
	end
end

function lineextradetails.onVehicleArrived(eventdata)
	local trace = false
	local tv = api.engine.getComponent(eventdata.id,  api.type.ComponentType.TRANSPORT_VEHICLE)
	if (tv.line == nil or tv.line < 1) then
		return
	end
	if (tv.stopIndex < 0) then
		return
	end

	local lineId = tv.line
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		if (trace) then
			_log("onVehicleArrived linedetails not set", eventdata.id)
		end
		return
	end

	if (linedetails.onVehicleArrived == nil) then
		if (trace) then
			_log("linedetails.onVehicleArrived not set", eventdata.id)
		end
		return
	end

	eventdata.line = tv.line
	eventdata.stopIndex = tv.stopIndex
	commonapi_gamescript.vehiclearrived[#commonapi_gamescript.vehiclearrived+1] = eventdata
end


function lineextradetails.onVehicleArrivedLater()
	local trace = false
	for idx, data in ipairs(commonapi_gamescript.vehiclearrived) do
		lineextradetails.runScriptAtEvent(data, "onVehicleArrived", trace)
	end
	commonapi_gamescript.vehiclearrived = {}
	return true
end

function lineextradetails.onVehicleArrived2Old(eventdata)
	local trace = false
	lineextradetails.runScriptAtEvent(eventdata, "onVehicleArrived", trace)
--[[
	if (data[uniqueStopId] ~= nil and data[uniqueStopId].setColor ~= nil) then
		math.randomseed(os.time())
		_log("linedetails.onVehicleArrived found setColor", eventdata.id)
		local tvW = commonapi._native.NewEventSystem.getComponentTransportVehicle(eventdata.id)
		local vehicles = tvW.transportVehicleConfig.vehicles
		for idx, vehicle in pairs(vehicles) do
			local color = vehicle.part.color;
			color.x = math.random(0, 10) * 0.1
			color.y = math.random(0, 10) * 0.1
			color.z = math.random(0, 10) * 0.1

			commonapi.dmp(color)
		end
		--local tvNeu = api.engine.getComponent(eventdata.id,  api.type.ComponentType.TRANSPORT_VEHICLE)
		return { updateColors = true }
	end
--]]
end

function lineextradetails.onLineVehicleSetNextStation(eventdata)
	local trace = false

	if (eventdata.line == nil) then
		return
	end
	local lineId = eventdata.line

	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		if (trace) then
			_log("onLineVehicleSetNextStation linedetails not set", eventdata.id, eventdata.line )
		end
		return
	end

	-- fast return path, if no settings are set (perf patch)
	if (linedetails.onVehicleNextStop == nil) then
		if (trace) then
			_log("onLineVehicleSetNextStation onVehicleNextStop not set", eventdata.id, eventdata.line )
		end
		return
	end
	if (trace) then
		commonapi.dmp(linedetails.onVehicleNextStop)
	end

	local tvOld = game.interface.getEntity(eventdata.id)

	if (tvOld.line == nil or tvOld.line < 0 or tvOld.stopIndex < 0) then
		return
	end

	local data = linedetails.onVehicleNextStop
	local uniqueIdList = lineextradetails.getUniqueIdsForLineStops(lineId)
	local uniqueStopId = uniqueIdList[tvOld.stopIndex+1] or "unknown"

	if (data[uniqueStopId] == nil) then
		return
	end

	local stopData = data[uniqueStopId]
	if (stopData.ifEmptyReturnTo ~= nil) then
		if (stopData.ifEmptyReturnTo == 0) then
			local cargoLoad = tvOld.cargoLoad or {}
			if (next(cargoLoad) == nil) then
				_log("onLineVehicleSetNextStation found empty vehicle with ifEmptyReturnTo rule", eventdata.id, eventdata.line )
				return {
					nextStation = 0
				}
			end
		end
	end
	eventdata.stopIndex = tvOld.stopIndex
	commonapi_gamescript.vehiclesetnextstop[#commonapi_gamescript.vehiclesetnextstop+1] = eventdata
end

function lineextradetails.onLineVehicleSetNextStationLater()
	local trace = false
	for idx, data in ipairs(commonapi_gamescript.vehiclesetnextstop) do
		lineextradetails.runScriptAtEvent(data, "onVehicleNextStop", trace)
	end
	commonapi_gamescript.vehiclesetnextstop = {}
	return true
end


function lineextradetails.verifyLines()
	if (datastorage.lines == nil) then
		return
	end

	local lineIds = {}
	for lineId, lineData in pairs(datastorage.lines) do
		lineIds[#lineIds+1] = lineId
	end

	for i, lineId in ipairs(lineIds) do
		lineextradetails.verifyLineData(lineId)
	end
end

function lineextradetails.verifyLineData(lineId)
	_log("verifyLineData("..lineId..")")

	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return
	end

	if (api.engine.entityExists(lineId) == false) then
		_log("verifyLineData() removing because line doesn't exists anymore", lineId)
		datastorage.lines[lineId] = nil
		return
	end

	local listOfStops, uniqueIdList = lineextradetails.getLineStops(lineId, false)

	local removeOldData = function(id)
		if (linedetails[id] == nil) then
			return
		end

		if (listOfStops == nil) then
			linedetails[id] = nil
			return
		end

		local remove = {}
		local t = linedetails[id]
		for key, _ in pairs(t) do
			if (uniqueIdList[key] == nil) then
				remove[#remove+1] = key
			end
		end
		for i = 1, #remove do
			local key = remove[i]
			_log("Removing non existent stop from line ", id, " ", key)
			t[key] = nil
		end

		if (next(t) == nil) then
			_log("verifyLineData() removing ", id)
			linedetails[id] = nil
		end
	end
	removeOldData("endStations")
	removeOldData("onEmptyVehicle")
	removeOldData("onVehicleArrived")

--[[
	if (linedetails.endStations ~= nil) then
		if (listOfStops == nil) then
			linedetails.endStations = nil
		else
			local remove = {}

			for key, _ in pairs(linedetails.endStations) do
				if (uniqueIdList[key] == nil) then
					remove[#remove+1] = key
				end
			end
			for i = 1, #remove do
				local key = remove[i]
				_log("Removing non existent stop from line", key)
				linedetails.endStations[key] = nil
			end
		end
		if (linedetails.endStations ~= nil and next(linedetails.endStations) == nil) then
			_log("verifyLineData() removing endStations")
			linedetails.endStations = nil
		end
	end

	if (linedetails.onEmptyVehicle ~= nil) then
		if (listOfStops == nil) then
			linedetails.onEmptyVehicle = nil
		else
			local remove = {}
			for key, _ in pairs(linedetails.onEmptyVehicle) do
				if (uniqueIdList[key] == nil) then
					remove[#remove+1] = key
				end
			end
			for i = 1, #remove do
				local key = remove[i]
				_log("Removing non existent stop from line onEmptyVehicle", key)
				linedetails.onEmptyVehicle[key] = nil
			end
		end

		if (linedetails.onEmptyVehicle ~= nil and next(linedetails.onEmptyVehicle) == nil) then
			_log("verifyLineData() removing onEmptyVehicle")
			linedetails.onEmptyVehicle = nil
		end
	end
--]]
	if (next(datastorage.lines[lineId]) == nil) then
		_log("verifyLineData() removing lines", lineId)
		datastorage.lines[lineId] = nil
	end
end

function lineextradetails.getUniqueIdsForLineStops(lineId)
	local lineComp = api.engine.getComponent(lineId, api.type.ComponentType.LINE)
	if (lineComp == nil) then
		return nil
	end

	local entityVisited = {}
	local getEntityIdx = function(entityId)
		if (entityVisited[entityId] == nil) then
			entityVisited[entityId] = 1
		else
			entityVisited[entityId] = entityVisited[entityId] + 1
		end
		return entityVisited[entityId]
	end
	local uniqueIdList = {}
	for stopIndex, stopData in ipairs(lineComp.stops) do
		local entityId = stopData.stationGroup
		local idx = getEntityIdx(entityId)
		local uniqueId = entityId..'-'..idx
		uniqueIdList[stopIndex] = uniqueId
	end

	return uniqueIdList
end

function lineextradetails.getLineStops(lineId, waypoints)
	local lineComp = api.engine.getComponent(lineId, api.type.ComponentType.LINE)
	if (lineComp == nil) then
		return nil
	end

	local entityVisited = {}
	local getEntityIdx = function(entityId)
		if (entityVisited[entityId] == nil) then
			entityVisited[entityId] = 1
		else
			entityVisited[entityId] = entityVisited[entityId] + 1
		end
		return entityVisited[entityId]
	end

	local listOfStops = {}
	local uniqueIdList = {}

	for stopIndex, stopData in ipairs(lineComp.stops) do
		local entityId = stopData.stationGroup
		local idx = getEntityIdx(entityId)
		local uniqueId = entityId..'-'..idx
		listOfStops[#listOfStops+1] = {
			id = entityId,
			idx = idx,
			uniqueId = uniqueId,
			lineId = lineId,
		}
		uniqueIdList[uniqueId] = true

		if (waypoints == true) then
			for waypointIndex, waypointEntityOrTable in ipairs(stopData.waypoints) do
				local waypointEntity
				if (type(waypointEntityOrTable) == 'userdata') then
					waypointEntity = waypointEntityOrTable.entity
				else
					waypointEntity = waypointEntityOrTable
				end
				local idxW = getEntityIdx(waypointEntity)
				local uniqueIdW = waypointEntity..'-'..idxW
				listOfStops[#listOfStops+1] = {
					id = waypointEntity,
					idx = idxW,
					uniqueId = uniqueIdW,
					lineId = lineId,
					waypoint = true,
				}
				uniqueIdList[uniqueIdW] = true
			end
		end
	end

	return listOfStops, uniqueIdList
end


function lineextradetails.getEndStations(lineId)
	if (datastorage.lines == nil) then
		return nil
	end
	if (datastorage.lines[lineId] == nil) then
		return nil
	end
	local linedetails = datastorage.lines[lineId]
	return linedetails.endStations
end




function lineextradetails.getEndStationData(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.endStations == nil) then
		return nil
	end
	return linedetails.endStations[uniqueId]
end


function lineextradetails.setEndStationData(lineId, uniqueId, data)
	local linedetails = lineextradetails.getOrCreateLineData(lineId)

	if (linedetails.endStations == nil) then
		linedetails.endStations = {}
	end
	linedetails.endStations[uniqueId] = data
	lineextradetails.syncData()
end


function lineextradetails.removeEndStationData(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.endStations ~= nil) then
		linedetails.endStations[uniqueId] = nil
	end
	lineextradetails.syncData()
end




function lineextradetails.getOnEmptyVehicleData(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.onEmptyVehicle == nil) then
		return nil
	end
	return linedetails.onEmptyVehicle[uniqueId]
end


function lineextradetails.setOnEmptyVehicleData(lineId, uniqueId, data)
	local linedetails = lineextradetails.getOrCreateLineData(lineId)

	if (linedetails.onEmptyVehicle == nil) then
		linedetails.onEmptyVehicle = {}
	end
	linedetails.onEmptyVehicle[uniqueId] = data
	lineextradetails.syncData()
end


function lineextradetails.removeOnEmptyVehicleData(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.onEmptyVehicle ~= nil) then
		linedetails.onEmptyVehicle[uniqueId] = nil
	end
	lineextradetails.syncData()
end


function lineextradetails.getOnVehicleArrived(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.onVehicleArrived == nil) then
		return nil
	end
	return linedetails.onVehicleArrived[uniqueId]
end

function lineextradetails.getOnVehicleArrivedAll(lineId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	return linedetails.onVehicleArrived
end

function lineextradetails.setOnVehicleArrived(lineId, uniqueId, data)
	local linedetails = lineextradetails.getOrCreateLineData(lineId)

	if (type(data) == "table" and next(data) ~= nil) then
		if (linedetails.onVehicleArrived == nil) then
			linedetails.onVehicleArrived = {}
		end
		_log("setOnVehicleArrived() add new data:", data)
		linedetails.onVehicleArrived[uniqueId] = data
	else
		if (linedetails.onVehicleArrived ~= nil) then
			linedetails.onVehicleArrived[uniqueId] = nil
		end
	end
--	commonapi.dmp(linedetails.onVehicleArrived)
	lineextradetails.syncData()
end



function lineextradetails.getOnVehicleNextStop(lineId, uniqueId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	if (linedetails.onVehicleNextStop == nil) then
		return nil
	end
	return linedetails.onVehicleNextStop[uniqueId]
end

function lineextradetails.getOnVehicleNextStopAll(lineId)
	local linedetails = lineextradetails.getLineData(lineId)
	if (linedetails == nil) then
		return nil
	end
	return linedetails.onVehicleNextStop
end

function lineextradetails.setOnVehicleNextStop(lineId, uniqueId, data)
	local linedetails = lineextradetails.getOrCreateLineData(lineId)

	if (type(data) == "table" and next(data) ~= nil) then
		if (linedetails.onVehicleNextStop == nil) then
			linedetails.onVehicleNextStop = {}
		end
		_log("setOnVehicleNextStop() add new data:", data)
		linedetails.onVehicleNextStop[uniqueId] = data
	else
		if (linedetails.onVehicleNextStop ~= nil) then
			linedetails.onVehicleNextStop[uniqueId] = nil
		end
	end
	lineextradetails.syncData()
end


-- Section Times --

function lineextradetails.calculateFIS()
	local gameTime = api.engine.getComponent(api.engine.util.getWorld(), api.type.ComponentType.GAME_TIME).gameTime

end



function lineextradetails.createVehicleData()

end
function lineextradetails.calculateArrivalData()
	local gameTime = api.engine.getComponent(api.engine.util.getWorld(), api.type.ComponentType.GAME_TIME).gameTime



	local vehicleData = {
		lineDestination = '',
		lineStopIdx = stopIdx,
		planedArrival = calculatedArrivalTime;
	}
	--lastLineStopDeparture = 
end




return lineextradetails



