local ModCommonListWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModCommonListWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

ModCommonListWindow = {}
ModCommonListWindow.caption = "ModCommonListWindow"


function ModCommonListWindow:checkExtractor(uilist)
	if (self.params.missingextractor == nil) then
		_log("Checking 7zip extractor")
		self.params.missingextractor_path = modmm.getExtractorBin()
		local zipok, zipmsg  = modmm.extractorIsUsable()
		self.params.missingextractor = not zipok
		self.params.missingextractor_msg = zipmsg
	end

	--if (self.params.missingextractor == true) then
		uilist[#uilist+1] = {
			typ = "Label",
			id = "extractorMissing",
			caption = _("7zip (${path}), ${msg}") % {
				path = self.params.missingextractor_path or "",
				msg = self.params.missingextractor_msg or ""
			},
		}
	if (self.params.missingextractor == true) then
		uilist[#uilist+1] = {
			typ = "SmallButton",
			id = "extractorMissingDoDownload",
			caption = _("Download 7zip"),
			sameline = true,
			onclick = function(item, win)
				local msg = _("Download 7zip?\n${url}") % { url = modmm.extractorGetDownloadURL() or "" }
				ui.msgbox("DoInstallExtractor", msg, "Download 7zip", {
					{ caption = _("Cancel") }, { caption = _("Download and extract..."), fn = function()
						local ok, errormsg = modmm.extractorDownload()
						if (ok == false) then
							ui.msgbox('ModListWindow_downloadExtractor', errormsg, "Download 7zip")
						end
						win.params.missingextractor = nil
						win:clearCache()
					end
				},
			})
			end
		}
	end
end

function ModCommonListWindow:genModUIList()
	local uilist = {}
	self.uilist = uilist

	if (self._modmmrepos == nil) then
		self._modmmrepos = modmm.getRepoDataObj()
		self._modmmrepos:reload()
	end


	return uilist
end


function ModCommonListWindow:genRepoDownloadList(uilist, repoentries)
	for mu, repoentry in ipairs( repoentries ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')  via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end

		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = repoentry.modid..'__repo__'..mu,
			caption = caption,
			data = {
				modid = repoentry.modid,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Install or Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..repoentry.modid.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genUpgradeList(uilist, entry)
	for mu, repoentry in ipairs( entry.repomods ) do

		local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
		if (repoentry.utc_changed ~= nil) then
			caption = caption .. '  ' .. os.date('%Y-%m-%d', repoentry.utc_changed)
		end
		uilist[#uilist+1] = {
			typ = "_Selectable",
			id = entry.id..'__repo__'..mu,
			caption = caption,
			data = {
				modid = entry.id,
				repoentry = repoentry,
			},
			onclick = function(item, win)
				--entry.downloadfull = repodata.download_prefix_url..''..entry.download
				--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
				--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
				ui.msgbox("DoModUpgrade",
					_("Install ${modname}?") % {
						modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
					}
					, "Mod upgrade", {
						{ caption = _("Cancel") }, { caption = _("Upgrade"), fn = function()
							local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
							if (ok == false) then
								ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..entry.id.." \n"..errormsg, "Update")
							end
							win:clearCache()
						end
					},
				})
			end,
		}
	end
end


function ModCommonListWindow:genModMMDownloadLink(uilist, repoentry)
	local caption = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')    via '..repoentry.repodata.name
	uilist[#uilist+1] = {
		typ = "Button",
		id = repoentry.modid..'__modmmdownload__',
		caption = caption,
		data = {
			repoentry = repoentry,
		},
		onclick = function(item, win)
			--entry.downloadfull = repodata.download_prefix_url..''..entry.download
			--entry.entryurlfull = repodata.entry_prefix_url..''..entry.entryurl
			--commonapi._native.utils.openurl(item.data.repodata.download_prefix_url..item.data.repoentry.download)
			ui.msgbox("DoModUpgrade",
				_("Install ${modname}?") % {
					modname = repoentry.name..' - ('..table.concat(repoentry.version, ".")..')'
				}
				, "Mod upgrade", {
					{ caption = _("Cancel") }, { caption = _("Upgrade or Downlad"), fn = function()
						local ok, errormsg = modmm.downloadAndInstallMod(item.data.repoentry)
						if (ok == false) then
							ui.msgbox('ModListWindow_downloadAndInstallMod', "Fehler beim Download/Update von:"..caption.." \n"..errormsg, "Update")
						end
						win:clearCache()
					end
				},
			})
		end,
	}
end


function ModCommonListWindow:genModSteamSubscribeLink(uilist, entry)
	local REGEX_STEAMIDFROMID = "^([0-9]+)_1$"
	local steamId = entry.id:match(REGEX_STEAMIDFROMID)
	local modName = entry.name or entry.id
	local caption = _("Subscribe to ${modName} (${steamId})") % {
		modName = modName,
		steamId = steamId
	}
	uilist[#uilist+1] = {
		typ = "Button",
		id = 'modsteamdownload__'..entry.id,
		caption = caption,
		data = {
			entry = entry,
		},
		onclick = function(item, win)
			if (item.data.downloaded == true) then
				return
			end
			if (commonapi._native and commonapi._native.dampf) then
				item.caption = _("Please wait")
				commonapi._native.dampf.subscribeItem(steamId, function(r)
					if (r == nil) then
						item.caption = "IOERROR"
						return
					end
					if (r.result == 1) then
						item.data.downloaded = true
						item.caption = _("Subscribed ${modName} done!") % {
							modName = modName,
							steamId = steamId,
							result = r.resultStr or r.result
						}
						item.typ = "Label"
						return
					end
					item.caption = _("${result} for ${modName} (${steamId})") % {
						modName = modName,
						steamId = steamId,
						result = r.resultStr or r.result
					}
				end)
			end
		end
	}
end

function ModCommonListWindow:genDepList(uilist, modentry)
	local requiredMods = modentry.data.info.requiredMods or {}

	for m, entry in pairs( requiredMods ) do

		local links = commonapi.mods._checkModDepends_createLinks(entry)

		local uimodid = {
			typ = "Label",
			caption = ""..(entry.modId or "No modId").." , "..(entry.steamId or "No SteamId")
		}
		uilist[#uilist+1] = uimodid
		local uiversion = {
			typ = "Label",
			caption = "\tMinorVersion: >="..(entry.minMinorVersion or 0)
		}
		uilist[#uilist+1] = uiversion

		if (entry.modId ~= nil and self._modmmrepos ~= nil) then
			local repoentries = self._modmmrepos:findEntriesByModId(entry.modId)
			if (repoentries ~= nil and #repoentries > 0) then
				ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
			end
		end

		for urltype, url in pairs( links ) do
			local urlentry = {
				typ = "Button",
				caption = urltype,
				sameline = true,
				id = ""..m..''..urltype,
				data = {
					myurl = url,
				},
				onclick = function(item, win)
					commonapi._native.utils.openurl(item.data.myurl)
				end
			}
			uilist[#uilist+1] = urlentry
		end
	end
end


function ModCommonListWindow:genErrorList(uilist, modentry)
	local mid = modentry.id..'__errors__'
	for i, error in ipairs( modentry.errors ) do
		local errstring = "?"
		if (error.type == "modmissing") then
			errstring = _("requires ${modId}") % {
				modId = (error.data.modId or error.data.steamId) or "?"
			}
		elseif (error.type == "modorder") then
			errstring = _("should be loaded after ${modname}") % {
				modname = error.mod.name
			}
			--errstring = "\t  ".. error.modname .. "should be loaded first"
		elseif (error.type == "modversion") then
			errstring = _("requires ${modname} ${version}") % {
				modname = error.mod.name,
				version = tostring(error.mod.version[1] or 0) .. "." .. error.data.minMinorVersion
			}
		elseif (error.type == "modinactive") then
			errstring = _("requires ${modname}, please activate") % {
				modname = error.mod.name,
			}
		else
			errstring = error.type
		end
		local uientry = {
			typ = "Label",
			caption = "\t"..errstring
		}
		uilist[#uilist+1] = uientry

		if (self._modmmrepos ~= nil and error.type == "modmissing" or error.type == "modversion") then
			if (error.data.modId ~= nil) then
				local repoentries = self._modmmrepos:findEntriesByModId(error.data.modId)
				if (repoentries ~= nil and #repoentries > 0) then
					ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
				end
			end

			if (error.links ~= nil) then
				for urltype, url in pairs( error.links ) do
					local urlentry = {
						typ = "Button",
						caption = urltype,
						sameline = true,
						id = mid..''..urltype,
						myurl = url,
						onclick = function(item, win)
							commonapi._native.utils.openurl(item.myurl)
						end
					}
					uilist[#uilist+1] = urlentry
				end
			end
		end
	end
end

function ModCommonListWindow:showRepositoryEditor(uilistparent)
	local repolist = self._modmmrepos:getRepoList()

	local uiTableData = {}
	local headerColumns = {
		{
			caption = "Filename",
			width = 100,
		},
		{ caption = "Url" },
		{
			caption =  _(""),
			width = 100,
		},
		{
			caption =  _("Last update"),
			width = 100,
		},
	}
	local TableHeader  = {
		typ = "TableHeader",
		items = headerColumns,
	}
	uiTableData[#uiTableData+1] = TableHeader

	uilistparent[#uilistparent+1] = {
		typ = "Table",
		id = "ModListWindowRepoListTable",
		columns = #headerColumns,
		border = false,
		uilist = uiTableData,
		size = {
			0, 100
		},
		flags = {
			Sortable = false,
			MultiSortable = false,
			ScrollFreezeTopRow = true
		}
	}


	for idx, repolistentry in ipairs(repolist) do
		local uientryFilename = {
			typ = "_Selectable",
			_fix_overlap = true,
			--id = repolistentry.filename..'__label',
			id = "itemRepoList_filename_"..idx,
			flags = {
				SpanAllColumns = true,
			},
			caption = repolistentry.filename,
			column = 0,
			rownew = true,
			onclick = function(item, win)
				win.data['curentryrepolist'] = repolistentry
			end,
			selected = function(item, win)
				return (win.data['curentryrepolist'] == repolistentry)
			end
		}
		uiTableData[#uiTableData+1] = uientryFilename

		local uientryURL = {
			typ = "Label",
			caption = repolistentry.url,
			column = 1,
		}
		uiTableData[#uiTableData+1] = uientryURL


		uiTableData[#uiTableData+1] = {
			typ = "SmallButton",
			id = "itemRepoList_Remove_"..idx,
			column = 2,
			caption = _("Remove"),
			data = {
				filename = repolistentry.filename,
			},
			onclick = function(item, win)
				self._modmmrepos:removeRepoFromList(item.data.filename)
				ModCommonListWindow.checkUpdates(win)
			end
		}


		local captionLastUpdate = ""
		
		if (repolistentry.lastupdate ~= nil) then
			captionLastUpdate = os.date('%Y-%m-%d %H:%M:%S', repolistentry.lastupdate)
		end

		local uientryLastUpdate = {
			typ = "Label",
			caption = captionLastUpdate,
			column = 3,
		}
		uiTableData[#uiTableData+1] = uientryLastUpdate
	end


	uiTableData[#uiTableData+1] = {
		typ = "TextInput",
		id = "addRepoFilename",
		column = 0,
		rownew = true,
		default = "name.repo"
	}

	uiTableData[#uiTableData+1] = {
		typ = "TextInput",
		id = "addRepoURL",
		column = 1,
		default = "https://",
	}

	uiTableData[#uiTableData+1] = {
		typ = "SmallButton",
		id = "addRepoList",
		column = 2,
		caption = _("Add"),
		onclick = function(item, win)
			local repourl = win.params['addRepoURL'] or "";
			if (repourl == "") then
				return
			end
			local repoFilename = win.params['addRepoFilename'] or "name.repo";
			self._modmmrepos:addRepoToList(repoFilename, repourl)
			ModCommonListWindow.checkUpdates(win)
		end
	}


	uilistparent[#uilistparent+1] = {
		typ = "SmallButton",
		id = "resetRepoList",
		caption = _("Reset"),
		onclick = function(item, win)
			self._modmmrepos:setRepoList()
			ModCommonListWindow.checkUpdates(win)
		end
	}



	uilistparent[#uilistparent+1] = {
		typ = "Separator",
	}


end

function ModCommonListWindow:checkUpdates()
	--local repositoryurl = "https://www.transportfever.net/filebase/repos/tpf2.json"
	local ok, errormsg = self._modmmrepos:updateRepoData()
	-- modmm.downloadRepository("tpfnet-tpf2.json", repositoryurl)
	if (ok == false) then
		ui.msgbox('ModCommonListWindow_checkUpdates', "Fehler beim Updaten: \n"..errormsg, "Download repository")
	end

	self:refresh()
end




function ModCommonListWindow:switchMods(switchentries)
	print("ModCommonListWindow:switchMods")
	local activemods = commonapi._native.ModList.getActive()
	for entryIndex, entry in ipairs(activemods) do
		if (entry.fullid == nil) then
			entry.fullid = entry.id.."_"..entry.version
		end
	end
	for idxS, switchentry in ipairs(switchentries) do
		local oldstate = -1
		for entryIndex, entry in ipairs(activemods) do
			if (entry.fullid == switchentry.missingId) then
				oldstate = entry.state
				entry.state = -1
			end
		end
		if (oldstate > -1) then
			for entryIndex, entry in ipairs(activemods) do
				if (entry.fullid  == switchentry.foundId) then
					entry.state = oldstate
				end
			end
		end
	end
	commonapi._native.ModList.setActive(activemods)
end


function ModCommonListWindow:enableMod(modid, position)
	print("ModCommonListWindow:enableMod")

	print(modid, position)
	local activemods = commonapi._native.ModList.getActive()
	local foundIdx = -1
	local maxstate = -1
	for entryIndex, entry in ipairs(activemods) do
		local fullid = entry.id.."_"..entry.version
		if (fullid == modid) then
			foundIdx = entryIndex
		end
		if (position ~= nil) then
			if (entry.state >= position) then
				entry.state = entry.state + 1
			end
		end

		if (maxstate < entry.state) then
			maxstate = entry.state
		end
	end
	if (foundIdx < 0) then
		return	-- not found?
	end

	if (position ~= nil) then
		activemods[foundIdx].state = position
	else
		activemods[foundIdx].state = maxstate + 1
	end
	commonapi._native.ModList.setActive(activemods)
end

function ModCommonListWindow:genSwitchModLink(uilist, switchentry)
	local caption = switchentry.modName
	uilist[#uilist+1] = {
		typ = "Button",
		id = switchentry.modId..'__modswitch__',
		caption = caption,
		data = {
			switchentry = switchentry,
		},
		onclick = function(item, win)
			ui.msgbox("DoModSwitch",
				_("Change ${modname}?") % {
					modname = item.data.switchentry.modName
				}
				, "Mod switch", {
					{ caption = _("Cancel") }, { caption = _("Switch to installed version"), fn = function()
						ModCommonListWindow.switchMods(win, { item.data.switchentry })
						win:clearCache()
					end
				},
			})
		end,
	}
end

function ModCommonListWindow:genEnableModLink(uilist, enableentry)
	uilist[#uilist+1] = {
		typ = "Button",
		id = enableentry.modId..'__modenable__'..enableentry.ownerModId,
		caption = _("Enable ${modname}") % {
			modname = enableentry.modName
		},
		onclick = function(item, win)
			ModCommonListWindow.enableMod(win, enableentry.modId, enableentry.ownerModIdx)
		end,
	}
end

---- requires a call to ModCommonListWindow.genModUIList(self) first
function ModCommonListWindow:createErrorList(uilist)
	local mods = commonapi.mods.getLoaded()
	for mid, entry in pairs( mods ) do
		if (entry.errors ~= nil) then
			if (#entry.errors > 0) then
				local uientry = {
					typ = "Label",
					caption = entry.name .. ":"
				}
				uilist[#uilist+1] = uientry
			end
			for i, error in ipairs( entry.errors ) do
				local errstring = "?"
				if (error.type == "modmissing") then
					errstring = _("requires ${modId}") % {
						modId = (error.data.modId or error.data.steamId) or "?"
					}
				elseif (error.type == "modorder") then
					errstring = _("should be loaded after ${modname}") % {
						modname = error.mod.name
					}
					--errstring = "\t  ".. error.modname .. "should be loaded first"
				elseif (error.type == "modversion") then
					errstring = _("requires ${modname} ${version}") % {
						modname = error.mod.name,
						version = tostring(error.mod.version[1] or 0) .. "." .. error.data.minMinorVersion
					}
				elseif (error.type == "modinactive") then
					errstring = _("requires ${modname}, please activate") % {
						modname = error.mod.name,
					}
				else
					errstring = error.type
				end
				local uientry = {
					typ = "Label",
					caption = "\t"..errstring
				}
				uilist[#uilist+1] = uientry


				if (self._modmmrepos ~= nil and error.data.modId ~= nil and (error.type == "modmissing" or error.type == "modversion")) then
					local repoentries = self._modmmrepos:findEntriesByModId(error.data.modId)
					if (repoentries ~= nil and #repoentries > 0) then
						ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
					end
				end
				if (error.data.steamId ~= nil and (error.type == "modmissing")) then
					if (commonapi._native and commonapi._native.dampf) then
						ModCommonListWindow.genModSteamSubscribeLink(self, uilist, {
							id = tostring(error.data.steamId).."_1"
						})
					end
				end


				if (error.links ~= nil) then
					for urltype, url in pairs( error.links ) do
						local urlentry = {
							typ = "Button",
							caption = urltype,
							sameline = true,
							id = mid..''..urltype,
							myurl = url,
							onclick = function(item, win)
								commonapi._native.utils.openurl(item.myurl)
							end
						}
						uilist[#uilist+1] = urlentry
					end
				end

				if (error.type == "modinactive") then
					local enableentry = {
						modId = error.mod._internalmodid,
						modName = error.mod.name,
						ownerModId = entry.id,
					}
					if (entry._idx ~= nil) then
						enableentry.ownerModIdx = entry._idx - 1
					end
					ModCommonListWindow:genEnableModLink(uilist, enableentry)
				end
			end
		end
	end


	local missingmods = commonapi.mods._getMissing()

	local steammodsmissing = {}
	local changeablemods = {}
	for mid, entry in pairs( missingmods ) do
		if (entry._isinactive ~= true) then

			local modentryX = commonapi.mods._getByIdAny(entry.id)

			if (modentryX == nil) then
				if (entry.name ~= nil and entry.name ~= "") then
					modentryX = commonapi.mods._getByNameAny(entry.name)
				end
			end
			if (modentryX ~= nil) then
				changeablemods[#changeablemods+1] = {
					modName = modentryX.name or modentryX.id,
					missingId = entry._internalmodid,
					foundId = modentryX._internalmodid,
					modId = modentryX._internalmodid
				}
			else
				local repoentries = self._modmmrepos:findEntriesByModId(entry.id)
				if (entry._installsource == "*") then
					ModCommonListWindow.genModSteamSubscribeLink(self, steammodsmissing, entry)
				elseif (repoentries ~= nil and #repoentries > 0) then
					local uimissing = {
						typ = "Label",
						caption = _("Missing ${modId}") % {
							modId = entry.id,
							modName = repoentries[1].name or "-"
						}
					}
					uilist[#uilist+1] = uimissing
					ModCommonListWindow.genModMMDownloadLink(self, uilist, repoentries[1])
				end
			end

		end
	end

	if (#changeablemods > 0) then
		uilist[#uilist+1] = {
			typ = "Label",
			caption = _("Switchable mods to installed version:")
		}

		uilist[#uilist+1] = {
			typ = "Button",
			id = "modswitch__all",
			caption = _("Change all"),
			styleclass = "info-message",
			data = {
				switchentries = changeablemods,
			},
			onclick = function(item, win)
				ui.msgbox("DoModSwitch",
					_("Change all mods tp installed versions?")
					, "Mod switch", {
						{ caption = _("Cancel") }, { caption = _("Switch"), fn = function()
							ModCommonListWindow.switchMods(win, item.data.switchentries )
							win:clearCache()
						end
					},
				})
			end,
		}
		for idx, switchentry in ipairs( changeablemods ) do
			ModCommonListWindow.genSwitchModLink(self, uilist, switchentry)
		end
	end

	if (#steammodsmissing > 0)  then
		uilist[#uilist+1] = {
			typ = "Label",
			caption = _("Steam mods missing:")
		}
		for idx, uientry in pairs( steammodsmissing ) do
			uilist[#uilist+1] = uientry
		end
	end
end




return ModCommonListWindow