local ModDownloadWindow = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.ModDownloadWindow")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


--commonapi._reloadModule("_modmm")
local modmm = commonapi._modmm;

local ModCommonListWindow = require("commonapi2.ui.ModCommonListWindow")

ModDownloadWindow = ui.Window:new()

ModDownloadWindow.id = "ModDownloadWindow"
ModDownloadWindow.height = 500
ModDownloadWindow.width = 800


ModDownloadWindow.caption = _("Mod download")
ModDownloadWindow.data = {
	modfilter = ""
}

function ModDownloadWindow.create()
	local o
	o = ModDownloadWindow:new()
	o:clearCache()	-- does genModUIList
	o:open()
	return o
end


function ModDownloadWindow:refresh()
	self:genModUIList()
end

function ModDownloadWindow:clearCache()
	commonapi.mods._loadByFS()
	self:genModUIList()
end

function ModDownloadWindow:installMod(filename)
	--commonapi.dmp(filename)
	--modmm.getFileInfo(filename)

	local ok, errormsg = modmm.installModFromArchiv(filename)
	if (ok == false) then
		ui.msgbox('ModListWindow:installMod', "Fehler beim installieren: \n"..errormsg, "Install Mod")
	end
end

function ModDownloadWindow:genModDetails(entry)
	if (self.uilist[#self.uilist].typ == "TabBar") then
		self.uilist[#self.uilist] = nil
	end
	local uiTabs = {}

	local uiTabBar = {
		typ = "TabBar",
		id = "ModDetailsTab",
		uilist = uiTabs
	}

	local uilistInfo ={}

	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = entry[1].name,
	}
	uilistInfo[#uilistInfo+1] = {
		typ = "Label",
		caption = table.concat(entry[1].version, "."),
	}

	--
	if (entry[1].entryurl ~= nil and entry[1].repodata ~= nil) then
		local entryfullurl = entry[1].repodata.entry_prefix_url..entry[1].entryurl
		uilistInfo[#uilistInfo+1] = {
			typ = "SmallButton",
			id = "ModDetailsTabInfoUrl",
			caption = entryfullurl,
			onclick = function(item, win)
				commonapi._native.utils.openurl(entryfullurl)
			end
		}
	end


	if (entry[1].isInstalled == true and entry[1].installedmod ~= nil) then
		uilistInfo[#uilistInfo+1] = {
			typ = "SmallButton",
			id = "ModDetailsTabInfoPath",
			caption = entry[1].installedmod.path,
			onclick = function(item, win)
				commonapi._native.utils.openfolder(entry[1].installedmod.path)
			end
		}
	end


	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabInfo",
		caption = "Details",
		uilist = uilistInfo,
	}


	local uilistUpgrade = {}
	uiTabs[#uiTabs+1] = {
		typ = "TabItem",
		id = "ModDetailsTabUpgrade",
		caption = _("Install or Upgrade"),
		uilist = {
			{
				typ = "ScrollRegion",
				id = "ScrollRegion",
				size = {
					0, 0,
				},
				uilist = uilistUpgrade,
			}
		},
	}
	
	--self:genDownloadList(uilistUpgrade, entry)
	ModCommonListWindow.genRepoDownloadList(self, uilistUpgrade, entry)
	if (entry[1].installedmod ~= nil) then
		local datainfo = entry[1].installedmod.data.info or {}
		local requiredMods = datainfo.requiredMods or {}
		if (#requiredMods > 0) then
			local uilistDeps = {}
			uiTabs[#uiTabs+1] = {
				typ = "TabItem",
				id = "ModDetailsTabDep",
				caption = _("Dependencies"),
				uilist = uilistDeps,
			}
			ModCommonListWindow.genDepList(self, uilistDeps, entry[1].installedmod)
			--self:genDepList(uilistDeps, entry[1].installedmod)
		end
	end


	if (entry[1].isInstalled == true) then
			if (entry[1].installedmod.errors ~= nil and #entry[1].installedmod.errors ~= 0) then

			local uilistErrors = {}
			uiTabs[#uiTabs+1] = {
				typ = "TabItem",
				id = "ModDetailsTabErrors",
				caption = _("Errors"),
				uilist = {
					{
						typ = "ScrollRegion",
						id = "ScrollRegion",
						size = {
							0, 0,
						},
						uilist = uilistErrors,
					}
				},
			}
			ModCommonListWindow.genErrorList(self, uilistErrors, entry[1].installedmod)
		end
	end

	self.uilist[#self.uilist+1] =  uiTabBar
end

local function compareversiontimestamp(a, b)
	local l = math.max(#a.version, #b.version)
	for i = 1, l do
		local ai = a.version[i] or 0
		local bi = b.version[i] or 0
		if (ai < bi) then
			return false
		end
		if (ai > bi) then
			return true
		end
	end

	local ac = a.utc_changed or 0
	local bc = b.utc_changed or 0
	if (ac > bc) then
		return true
	end
	return false
end


function ModDownloadWindow:genRepoModUIList_CreateRow(uilist, repoentry)
	if (#repoentry < 1) then
		return
	end

	local entry = repoentry[1]
	local entryid = entry.modid or "?"
	local entryname = entry.name or entryid

	--local hasUpgrade = (entry.upgradeversion ~= nil)
	local uientry = {
		--typ = "Label",
		typ = "_Selectable",
		_fix_overlap = true,
		id = entryid..'__label',
		flags = {
			SpanAllColumns = true,
		},
		caption = entryname,
		column = 0,
		rownew = true,
		onclick = function(item, win)
			win.data['curentry'] = repoentry
			win:genModDetails(repoentry)
		end,
		selected = function(item, win)
			return (win.data['curentry'] == repoentry)
		end
	}
	uilist[#uilist+1] = uientry

	if (entry.isUpgrade == true) then
		uientry.color = { 1.0, 1.0, 0, 1.0 }
	end

	if (entry.isInstalled == true) then
		if (entry.installedmod.errors ~= nil and #entry.installedmod.errors ~= 0) then
			uientry.color = { 1.0, 0.0, 0.0, 1.0 }
		end
	end


	local uientryId = {
		typ = "Label",
		caption = entryid,
		column = 1
	}
	uilist[#uilist+1] = uientryId


	local uientryVersion = {
		typ = "Label",
		caption = table.concat(entry.version, "."),
		column = 2,
	}
	uilist[#uilist+1] = uientryVersion

	local uientryInstVersion = {
		typ = "Label",
		caption = table.concat(entry.installedmod_version, "."),
		column = 3,
	}

	uilist[#uilist+1] = uientryInstVersion


	local uientryHasUpgrade = {
		typ = "Label",
		caption = "",
		column = 4
	}
	if (entry.isUpgrade == true) then
		uientryHasUpgrade.caption = "*"
	end
	uilist[#uilist+1] = uientryHasUpgrade
--[[
	local uientryButton = {
		typ = "SmallButton",
		id = entryid..'_install',
		caption = _("Install"),
		column = 4,
		data = {
			modid = entryid,
		},
		onclick = function(item, win)
			--local o = ModSettingsWindow.create(item.data.modid)
			--o:open()
		end
	}
	uilist[#uilist+1] = uientryButton
--]]
end

function ModDownloadWindow:genModUIList()
	local uilist  = ModCommonListWindow.genModUIList(self)
	ModCommonListWindow.checkExtractor(self, uilist)

	local repo = {}

	local selectedrepoidx = self.params.selectedrepoidx or 0
	local allrepos = self._modmmrepos:getLoadedRepositories()

	if (allrepos[selectedrepoidx+1] ~= nil) then
		repo = allrepos[selectedrepoidx+1] or nil
	end

	if (repo == nil) then
		repo = {}
	end
	local repomods = repo.mods or {}

	if (commonapi.mods._isEmpty()) then
		commonapi.mods._loadByFS()
	end

	self.data.repotimestamp = self._modmmrepos:getLastRepoDataUpdate()

	uilist[#uilist+1] = {
		typ = "Button",
		id = "checkUpdates",
		caption = _("Download update information"),
		--sameline = true,
		onclick = function(item, win)
			ModCommonListWindow.checkUpdates(win)
		end
	}

	if (self.data.repotimestamp ~= nil) then
		uilist[#uilist+1] = {
			typ = "Label",
			caption = os.date('%Y-%m-%d %H:%M:%S', self.data.repotimestamp),
			sameline = true,
		}
	end


	uilist[#uilist+1] = {
		typ = "Checkbox",
		id = "modifyRepositoryList",
		caption = _("Repositories"),
		sameline = true,
		onclick = function(item, win)
			win:refresh()
		end
	}

	if (self.params["modifyRepositoryList"] == true) then
		ModCommonListWindow.showRepositoryEditor(self, uilist)
	end


	if (#allrepos > 0) then
		local reponames = {}
		for idx, reposentry in ipairs(allrepos) do
			reponames[#reponames+1] = reposentry.repo.name.." "..os.date('%Y-%m-%d %H:%M:%S', tonumber(reposentry.repo.utc_time) or 0)
		end
		uilist[#uilist+1] =	{
			typ = "Buttons",
			id = "selectedrepoidx",
			values = reponames,
			defaultIndex = 0,
			onclick = function(item, win)
				win:refresh()
			end
		}
	end

	uilist[#uilist+1] = {
		typ = "TextInput",
		id = "modfilter",
		changeonenter = true,
		onchange = function(item, win)
			win:refresh()
		end
	}

	local uiTableData = {}

	local headerColumns = {
		{ caption = "Name" },
		{ caption = "Modid" },
		{
			caption = "Version",
			width = 50,
		},
		{
			caption = "Inst. Version",
			width = 60,
		},
---[[
		{
			caption = _("U"),
			width = 50,
		}
--]]
	}
	local TableHeader  = {
		typ = "TableHeader",
		items = headerColumns,
		onchange = function(item, win)
			win.cursorting = item.cursorting
			win:refresh()
		end
	}
	uiTableData[#uiTableData+1] = TableHeader

	local repomods_ids = {}
	local i = 1


	local filter = string.lower(self.params.modfilter or "")

	for k, v in pairs( repomods ) do
		table.sort(v, compareversiontimestamp)

		local firstentry = v[1]
		local installedmod = commonapi.mods._getByIdAny(firstentry.modid)

		if (installedmod ~= nil) then
			firstentry.isInstalled = true
			firstentry.installedmod = installedmod
			firstentry.installedmod_version = installedmod.version
		--	commonapi.dmp(installedmod)
		--	commonapi.dmp(firstentry)
		--	commonapi.dmp(_utils.compareVersion(installedmod.version, firstentry.version))
			if (_utils.compareVersion(installedmod.version, firstentry.version) == -1) then
				firstentry.isUpgrade = true
			else
				firstentry.isUpgrade = false
			end
		else
			firstentry.installedmod_version = {}
			firstentry.installedmod = nil
			firstentry.isInstalled = false
			firstentry.isUpgrade = false
		end


		local name_lower = string.lower(firstentry['name'])
		local modid_lower = string.lower(firstentry['modid'])
		firstentry['name_lower'] = name_lower
		firstentry['modid_lower'] = modid_lower
		if (filter ~= '') then
			if ( 	(string.find( firstentry['name_lower'], filter) ~= nil) or 
					(string.find( firstentry['modid_lower'], filter) ~= nil) ) then
				repomods_ids[i] = k
				i = i + 1
			end
		else
			repomods_ids[i] = k
			i = i + 1
		end
	end

	if (self.cursorting ~= nil) then
		local sortspecs = self.cursorting.specs or {}
		local comparestring = function(a, b)
			--local a = string.lower(as)
			--local b = string.lower(bs)
			if (a < b) then
				return -1
			end
			if (a > b) then
				return 1
			end
			return 0
		end


		local compareBool = function(a, b)
			if (not a and b) then
				return -1
			end
			if (a and not b) then
				return 1
			end
			return 0
		end

		local fnbycolumn = {
			comparestring,
			comparestring,
			_utils.compareVersion,
			_utils.compareVersion,
			compareBool
		}

		local keybycolumn = {
			"name_lower",
			"modid_lower",
			"version",
			"installedmod_version",
			"isUpgrade",
			"",
		}
		local sortfn = function(aid, bid)
			local a = repomods[aid][1]
			local b = repomods[bid][1]
			--commonapi.dmp(a)
			--commonapi.dmp(b)
			for k, sortspec in ipairs(sortspecs) do
				local key = keybycolumn[sortspec.columnIndex+1] or ""
				if (key ~= "") then
		
					local fn = fnbycolumn[sortspec.columnIndex+1] or nil
					local delta = fn(a[key], b[key])
					if (delta > 0) then
						return (sortspec.sortDirection == 2)
					elseif (delta < 0) then
						return (sortspec.sortDirection == 1)
					end
				end
			end

			return false
		end
		table.sort(repomods_ids, sortfn)
	end

	for _, repoentryid in pairs( repomods_ids ) do
		local repoentry = repomods[repoentryid]
		self:genRepoModUIList_CreateRow(uiTableData, repoentry)
	end

	uilist[#uilist+1] = {
		typ = "Table",
		id = "ModDownloadTable",
		columns = #headerColumns,
		border = false,
		uilist = uiTableData,
		size = {
			0, -100
		},
		flags = {
			Sortable = true,
			MultiSortable = true,
			ScrollFreezeTopRow = true
		}
	}

	uilist[#uilist+1] = {
		typ = "Separator"
	}
	self.uilist = uilist
end

return ModDownloadWindow