local csc = {}
local modmm = commonapi._modmm;
local ui = commonapi.ui;

local pkgname = "commonapi2.ui2.CustomSettingsCompOverlay"


local ModCommonListWindow = require("commonapi2.ui.ModCommonListWindow")


local cscDialog = ui.UIContainer:define()
cscDialog.id = "cscDialog"

function cscDialog:refresh()
	self:createUIList()
	self:rebuild()
end

function cscDialog:msgbox(id, text, caption, buttons)
	self.text = text or ""
	self.caption = caption or "CommonAPI2 DialogBox"
	self.buttons = buttons
	self:refresh()
	self._ugwindow:setVisible(true, true)
end

function cscDialog:close()
	self._ugwindow:setVisible(false, true)
end

function cscDialog:createUIList()
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		name = "DialogBox::Title",
		caption = (self.caption or "CommonAPI2 DialogBox")
	}

	uilist[#uilist+1] = {
		typ = "Label",
		name = "DialogBox::Text",
		caption = (self.text or "")
	}
	local uilistbuttons = {}
	uilist[#uilist+1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttons
	}



	if (self.buttons == nil) then
		uilistbuttons[#uilistbuttons+1] = {
			typ = "Button",
			caption = self._("OK"),
			id = "ok",
			onclick = function(item, win)
				win:close()
			end
		}
	else
		for i, entry in ipairs( self.buttons ) do
			uilistbuttons[#uilistbuttons+1] = {
				typ = "Button",
				id = "dialogbutton"..i,
				--entry['caption'],
				caption = entry['caption'],
				data = {
					fn = entry['fn'] or false
				},
				onclick = function(item, win)
					win:close()
					if (item.data.fn) then
						item.data.fn()
					end
				end
			}
		end
	end

	self.uilist = uilist
end


local cscui = ui.UIContainer:define()
cscui.id = "cscui"

function cscui:refresh()
	self:createUIList()
	self:rebuild()
end

function cscui:clearCache()
	self:refresh()
end




function cscui:createUIList()
	local uilist = {}

	ModCommonListWindow.genModUIList(self)

	uilist[#uilist+1] = {
		typ = "Label",
		caption = "CommonAPI2 result:"
	}
	local uilistregion = {}
	uilist[#uilist + 1] = {
		typ = "ScrollRegion",
		id = "ScrollRegion",
		border = true,
		size = {
			0,
			100,
		},
		uilist = uilistregion,
		gravity = { -1, 0 }
	}
	ModCommonListWindow.createErrorList(self, uilistregion)

	self.uilist = uilist
end


function csc.handleModListChange(data)
	print("csc.handleModListChange")

	commonapi.mods._reset()
	if (type(data) == "table") then
		local sfn = function(a, b)
			return a.state < b.state
		end
		table.sort(data, sfn)
		for i, entry in ipairs(data) do
			if (entry.isMissing == true) then
				commonapi.mods._addMissingModId(entry.id.."_"..entry.version, entry.name or nil, (entry.state < 0))
			else
				commonapi.mods._addModByPathAndId(entry.path, entry.id.."_"..entry.version, (entry.state < 0) )
			end
		end
	end
	commonapi.mods._checkMods()

	if (csc.uicontainer ~= nil) then
		csc.uicontainer:refresh()
	end
end

function csc.destroyIt()
	local container = api.gui.util.getById("CommonAPI2ModListContainer")
	if (container ~= nil) then
		local parent = container:getParent()
		local layout = parent:getLayout()
		layout:removeItem(container)
		container:destroy()
	end

	local containerDialog = api.gui.util.getById("CommonAPI2ModListDialogBox")
	if (containerDialog ~= nil) then
		local parent = containerDialog:getParent()
		local layout = parent:getLayout()
		-- Note: Parent is a CFloatingLayout, so getParent doesn't work
		local floatingLayout = layout:getItem(0)
		floatingLayout:removeItem(containerDialog)
		containerDialog:destroy()
	end
	ui.msgbox = nil
end

function csc.createBaseUI(customsettingscomp)
	csc.customsettingscomp = customsettingscomp
	-- Debug:
	csc.customsettingscomp:setId("customsettingscomp")
	csc.container = nil
	csc.containerLayout = nil

	local l = csc.customsettingscomp:getLayout();

	local container = api.gui.comp.Component.new("CommonAPI2ModListContainer")
	container:addStyleClass("tab-widget-content")
	ui.forceId(container, "CommonAPI2ModListContainer")
	local containerLayout = api.gui.layout.BoxLayout.new("VERTICAL")
	csc.containerLayout = containerLayout
	container:setLayout(containerLayout)

	l:addItem(container);

	csc.uicontainer = cscui:create(container)
	csc.uicontainer:refresh()


	-- Note: floatingLayout is a CFloatingLayout Container, so destructor is different in destroyIt
	local floatingLayout = l:getItem(0)
	local dialogContainer = api.gui.comp.Component.new("DialogBox")
	--dialogContainer:addStyleClass("load-save-preset")
	ui.forceId(dialogContainer, "CommonAPI2ModListDialogBox")

	local dialogContainerLayout = api.gui.layout.FloatingLayout.new()
	dialogContainer:setLayout(dialogContainerLayout)

	local maxsize = api.gui.util.Size.new( 500, 500 )
	dialogContainer:setMaximumSize(maxsize)
	local dialogboxOverlay = api.gui.comp.Component.new("DialogBox::Overlay")
	local dialogboxContent =  api.gui.comp.Component.new("DialogBox::Content")
	local dialogLayout = api.gui.layout.BoxLayout.new("VERTICAL")
	dialogboxContent:setLayout(dialogLayout)
	dialogboxOverlay:setGravity(-1, -1)
	dialogboxContent:setGravity(-1, -1)
	dialogContainerLayout:addItem(dialogboxOverlay, 1, 1)
	dialogContainerLayout:addItem(dialogboxContent, 1, 1)

	dialogContainer:setVisible(false, false)
	floatingLayout:addItem(dialogContainer, 0.5, 0.5)
	--l:addItem(dialogContainer);

	csc.uidialog = cscDialog:create(dialogboxContent)
	csc.uidialog:refresh()

	ui.msgbox = function(id, text, caption, buttons)
		csc.uidialog:msgbox(id, text, caption, buttons)
	end
end


function csc.reload()
	package.loaded[pkgname] = nil
	local cscNew = require(pkgname)
	local customsettingscomp = csc.customsettingscomp
	cscNew.init(customsettingscomp)
end

function csc.init(comp)
	csc.createBaseUI(comp)

	commonapi._native.ModList.setChangeCallback(csc.handleModListChange)

	csc._modmmrepos = modmm.getRepoDataObj()
	csc._modmmrepos:reload()
end


return csc