local LSS = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.ui2.linestopsettings")
local ui = require("commonapi2.ui2.tpfui")

local lineextradetails = require("commonapi2.lineextradetails.lineextradetails")

LSS = ui.Window:new()

LSS.id =  "LSS"
LSS.caption = "Line stop settings"
LSS.uilist = {}

LSS.height = 500
LSS.width = 500

local function syntaxCheck(script)
	local vs
	local vsok, vsobj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.vehiclescript")
	if (vsok == true and type(vsobj) == "table") then
		vs = vsobj
	else
		return false, "Can't load vehiclescript: "..vsobj, nil
	end
	local isok, err = vs.run(-1, script, "syntaxCheck")
	if (isok == false) then
		local linenr = string.match(err, "%[string \"syntaxCheck\"%]:(%d+)")
		if (linenr ~= nil) then
			linenr = tonumber(linenr)
		end
		return false, string.gsub(err, "%[string \"syntaxCheck\"%]:", "Line: "), linenr
	end
	return true;
end

function LSS.create(lineId, stopUniqueId, stopData)
	local o
	o = LSS:new()
	o.lineEntity = lineId
	o.stopUniqueId = stopUniqueId
	o.stopData = stopData
	--o.id =  "LSS-"..lineId.."-"..stopUniqueId

	local lineNameComp = api.engine.getComponent(lineId, api.type.ComponentType.NAME)
	local stopdNameComp = api.engine.getComponent(stopData.id, api.type.ComponentType.NAME)
	o.caption = "LSS "..lineNameComp.name.."("..lineId..")  "..stopdNameComp.name.." ("..stopUniqueId..")"

	o:createUIList()
	o:open()
	return o
end

function LSS:refresh()
	self:createUIList()
	self:rebuild()
end

local function splitnewlines(str)
	if string.find(str, "\n") == nil then
		return { str }
	end

	local lines = {}
	local pat = "(.-)\r?\n()"
	local lastPos
	for part, pos in string.gmatch(str, pat) do
		lines[#lines+1] = part
		lastPos = pos
	end
	lines[#lines+1] = string.sub(str, lastPos)
	return lines
end

function LSS:changeSettings()
	local onVehicleArrived = lineextradetails.getOnVehicleArrived(self.stopData.lineId, self.stopData.uniqueId)
	if (onVehicleArrived == nil) then
		onVehicleArrived = {}
	end
	if (self.params["onVehicleArrived_runScript"] == true) then
		print("onVehicleArrived.runScript adding")
		onVehicleArrived.runScript = true
	else
		onVehicleArrived.runScript = nil
	end

	if (self.params["onVehicleArrived_runScriptSrc"] ~= "") then
		onVehicleArrived.runScriptSrc = self.params["onVehicleArrived_runScriptSrc"]
	else
		onVehicleArrived.runScriptSrc = nil
	end

	lineextradetails.setOnVehicleArrived(self.stopData.lineId, self.stopData.uniqueId, onVehicleArrived)

	local onVehicleNextStop= lineextradetails.getOnVehicleNextStop(self.stopData.lineId, self.stopData.uniqueId)
	if (onVehicleNextStop == nil) then
		onVehicleNextStop = {}
	end

	if (self.params["onVehicleNextStop_ifEmptyReturnTo"] == true) then
		onVehicleNextStop.ifEmptyReturnTo = 0
	else
		onVehicleNextStop.ifEmptyReturnTo = nil
	end

	if (self.params["onVehicleNextStop_runScript"] == true) then
		print("onVehicleNextStop.runScript adding")
		onVehicleNextStop.runScript = true
	else
		onVehicleNextStop.runScript = nil
	end

	if (self.params["onVehicleNextStop_runScriptSrc"] ~= "") then
		onVehicleNextStop.runScriptSrc = self.params["onVehicleNextStop_runScriptSrc"]
	else
		onVehicleNextStop.runScriptSrc = nil
	end
	lineextradetails.setOnVehicleNextStop(self.stopData.lineId, self.stopData.uniqueId, onVehicleNextStop)

	local LineDestination = require("commonapi2.ui2.LineDestination")
	LineDestination:updateUI()
end


function LSS:createUIList()
	local onVehicleArrived = lineextradetails.getOnVehicleArrived(self.stopData.lineId, self.stopData.uniqueId)

	if (onVehicleArrived == nil) then
		onVehicleArrived = {}
	end

	local onVehicleNextStop = lineextradetails.getOnVehicleNextStop(self.stopData.lineId, self.stopData.uniqueId)

	if (onVehicleNextStop == nil) then
		onVehicleNextStop = {}
	end

	self.params = {}
	self.params["onVehicleArrived_runScriptSrc"] = onVehicleArrived.runScriptSrc or ""
	self.params["onVehicleArrived_runScript"] = onVehicleArrived.runScript or false

	self.params["onVehicleNextStop_ifEmptyReturnTo"] = (onVehicleNextStop.ifEmptyReturnTo == 0) or false
	self.params["onVehicleNextStop_runScriptSrc"] = onVehicleNextStop.runScriptSrc or ""
	self.params["onVehicleNextStop_runScript"] = onVehicleNextStop.runScript or false

	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = _utils._t("On vehicle arrived:")
	}

	uilist[#uilist + 1] = {
		typ = "Checkbox",
		id = "onVehicleArrived_runScript",
		caption = "Run script",
		onclick = function(item, win)
		--	local cursettings = commonapi._native.UIHacks.getRenderSettings( api.gui.util.getGameUI() )
		--	cursettings[name] = win.params[item.id]
			win:changeSettings()
			win:refresh()
		end
	}

	self:createUIScript(uilist, "onVehicleArrived_runScriptSrc", self.params["onVehicleArrived_runScriptSrc"] )
	--self:createUIListColor(uilist, "onVehicleArrived_color", palette)



	uilist[#uilist+1] = {
		typ = "Label",
		caption = _utils._t("On vehicle set next stop:")
	}

	uilist[#uilist + 1] = {
		typ = "Checkbox",
		id = "onVehicleNextStop_ifEmptyReturnTo",
		caption = "On empty cargo return to first stop",
		onclick = function(item, win)
			win:changeSettings()
			win:refresh()
		end
	}

	uilist[#uilist + 1] = {
		typ = "Checkbox",
		id = "onVehicleNextStop_runScript",
		caption = "Run script",
		onclick = function(item, win)
		--	local cursettings = commonapi._native.UIHacks.getRenderSettings( api.gui.util.getGameUI() )
		--	cursettings[name] = win.params[item.id]
			win:changeSettings()
			win:refresh()
		end
	}
	self:createUIScript(uilist, "onVehicleNextStop_runScriptSrc", self.params["onVehicleNextStop_runScriptSrc"] )

	uilist[#uilist + 1] = {
		typ = "Button",
		id = "ScriptHelpButton",
		caption = "Script help",
		onclick = function(item, win)
			package.loaded["commonapi2.ui2.ScriptHelp"] = nil
			--package.loaded["commonapi2.ui2.tpfui"] = nil
			local ok, ScriptHelp = pcall(require,  "commonapi2.ui2.ScriptHelp")
			if (ok == true) then
				local okcreate, error = pcall(ScriptHelp.create)
				if (okcreate == false) then
					print(error)
				end
			else
				print(ScriptHelp)
			end
		end
	}
	self.uilist = uilist
end


function LSS:createUIScript(uiparent, id, scriptsrc)
	local uiTableData = {}


	local syntaxmsg
	local scriptok, scripterr, lineerr = syntaxCheck(scriptsrc)
	if (scriptok) then
		syntaxmsg = "Syntax OK"
	else
		syntaxmsg = scripterr
	end

	local headerColumns = {
		{
			caption = "LineNr.",
			width = 10
		},
		{
			caption = "Script",
		},
		{
			caption = "",
			width = 10
		},
		{
			caption = "",
			width = 10
		}
	}
	local TableHeader = {
		typ = "TableHeader",
		items = headerColumns,
	}
	uiTableData[#uiTableData + 1] = TableHeader

	uiparent[#uiparent + 1] = {
		typ = "Table",
		id = id,
		columns = #headerColumns,
		border = true,
		uilist = uiTableData,
		flags = {
			Sortable = false,
			MultiSortable = false,
			ScrollFreezeTopRow = true
		}
	}


	local lines = splitnewlines(scriptsrc)
	local maxIdx = #lines

	local function mergeLines(win)
		local newLine = ""
		for idx = 1, maxIdx do
			local fid = id.."-"..idx
			if (win.params[fid] ~= nil) then
				if (idx ~= maxIdx) then
					newLine = newLine..self.params[fid].."\n"
				else
					newLine = newLine..self.params[fid]
				end
			else
				--print("skipping ", fid)
			end
		end
		return newLine
	end

	for idx, line in ipairs(lines) do
		local fid = id.."-"..idx
		self.params[fid] = line
		local labelitem =  {
			typ = "Label",
			caption = ""..idx,
			rownew = true,
			column = 0
		}

		if (idx == lineerr) then
			labelitem.styleclass = "warning"
			labelitem.caption = labelitem.caption .. " !>"
		end

		uiTableData[#uiTableData + 1] = labelitem

		uiTableData[#uiTableData + 1] = {
			typ = "TextInput",
			id = fid,
			gravity = { -1, 0 },
			changeonenter = true,
			onchange = function(item, win)
				win.params[id] = mergeLines(win)
				win:changeSettings()
				win:refresh()
			end,
			column = 1,
			styleclass = "commonapi-script"
		}

		uiTableData[#uiTableData + 1] = {
			typ = "Button",
			id = fid.."_remove",
			fid = fid,
			caption = "x",
			column = 2,
			onclick = function(item, win)
				--print("Trying to remove line ", item.fid)
				win.params[item.fid] = nil

				--commonapi.dmp(win.params["onVehicleArrived_runScriptSrc"])
				win.params[id] = mergeLines(win)
				--commonapi.dmp(win.params["onVehicleArrived_runScriptSrc"])
				win:changeSettings()
				win:refresh()
			end
		}

		uiTableData[#uiTableData + 1] = {
			typ = "Button",
			id = fid.."_add",
			fid = fid,
			caption = "+",
			column = 3,
			onclick = function(item, win)
				--print("Trying to add line ", item.fid)
				win.params[item.fid] = win.params[item.fid].."\n"

				--commonapi.dmp(win.params["onVehicleArrived_runScriptSrc"])
				win.params[id] = mergeLines(win)
				--commonapi.dmp(win.params["onVehicleArrived_runScriptSrc"])
				win:changeSettings()
				win:refresh()
			end
		}
	end

	local uilistbuttons = {}
	uiparent[#uiparent + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttons,
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = id.."Copy",
		caption = self._("Copy"),
		onclick = function(item, win)
			commonapi._native.utils.setClipboardText(mergeLines(win))
		end
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = id.."Paste",
		caption = self._("Paste"),
		onclick = function(item, win)
			local txt = commonapi._native.utils.getClipboardText()
			if (txt == "") then
				return
			end
			win.params[id] = txt
			win:changeSettings()
			win:refresh()
		end
	}

	uiparent[#uiparent + 1] = {
		typ = "Label",
		caption = syntaxmsg,
	}
end

function LSS:createUIListColor(uiparent, id, palette)
	local uiTableData = {}

	local headerColumns = {
		{
			caption = "Color",
			width = 50
		},
		{
			caption ="Variation",
			width = 50
		},
		{
			caption = "Remove"
		}
	}
	local TableHeader = {
		typ = "TableHeader",
		items = headerColumns,
		onchange = function(item, win)
			--win.cursorting = item.cursorting
			--win:refresh()
		end
	}
	uiTableData[#uiTableData + 1] = TableHeader

	uiparent[#uiparent + 1] = {
		typ = "Table",
		id = id,
		columns = #headerColumns,
		border = true,
		uilist = uiTableData,
		size = {
			0,
			100
		},
		flags = {
			Sortable = false,
			MultiSortable = false,
			ScrollFreezeTopRow = true
		},
		onclick = function(item, win)
			--[[
			local index = win.params["EntitiesInZoneTable"]
			if (index >= 0 and self._zoneEntities) then
				local entityid = self._zoneEntities[index + 1]
				if (entityid) then
					win.params.EntityId = tostring(entityid)
				end
			end
			win:refresh()
			--]]
		end
	}
	local lastindex = -1
	for idx, color in pairs(palette) do
		local itemid = id.."_"..idx
		self.params[itemid] = palette.color
		uiTableData[#uiTableData + 1] = {
			typ = "ColorChooserButton",
			id = itemid,
			onchange = function(item, win)
				print(idx)
				commonapi.dmp( win.params[item.id] )
				win:refresh()
			end,
			rownew = true,
			column = 0
		}

		uiTableData[#uiTableData + 1] = {
			typ = "Label",
			caption = "x",
			column = 1
		}

		uiTableData[#uiTableData + 1] = {
			typ = "Label",
			caption = "x2",
			column = 2
		}
		lastindex = idx
	end
end

return LSS