local LoadErrorsWindow = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.ui2.LoadErrorsWindow")
local ui = require("commonapi2.ui2.tpfui")
local checkModels = require("commonapi2.fix.checkModels")

LoadErrorsWindow = ui.Window:new()

LoadErrorsWindow.id =  "LoadErrors"
LoadErrorsWindow.caption = "Load Errors"
LoadErrorsWindow.uilist = {}

LoadErrorsWindow.height = 500
LoadErrorsWindow.width = 900
LoadErrorsWindow.forceSize = true


function LoadErrorsWindow.create()
	local o
	o = LoadErrorsWindow:new()
	o:createUIList()
	o:open()
	o:setPositionCenter()
	return o
end

function LoadErrorsWindow:refresh()
	self:createUIList()
	self:rebuild()
end

function LoadErrorsWindow:createUIList()
	local uilist = {}
	local uitabs = {}

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "ModelErrorsTab",
		caption = "Model Errors",
		uilist = self:createUIListModelErrors()
	}

	uilist[#uilist + 1] = {
		typ = "TabBar",
		id = "LoadErrorsTabs",
		default = 0,
		uilist = uitabs
	}

	self.uilist = uilist
end

function LoadErrorsWindow:copyToClipboard(fileName)
	local modelErrors = checkModels.getErrors()
	local d = modelErrors[fileName]
	local str = {}
	str[#str+1] = "CommonAPI2 checkModels report for \n"..fileName.."\n"
	if (d.lines ~= nil) then
		for idx, line in ipairs(d.lines) do
			str[#str+1] = "      "..(line.metadatapath or "")..": "
			str[#str+1] = line.failure or line.text

			if (line.lodId ~= nil) then
				if (line.lodId > 0) then
					str[#str+1] =  "  LOD"..(line.lodId-1)
				end
			end
			if (line.severity ~= nil) then
				str[#str+1] = "   (severity: "..line.severity..")"
			end
			str[#str+1] = "\n"
		end
	end

	local msg = table.concat(str, "")
	if (commonapi._native ~= nil) then
		commonapi._native.utils.setClipboardText(msg)
	else
		print(msg)
	end
end

function LoadErrorsWindow:createUIListModelErrors()
	local uilist = {}
	local uilistscroll = {}
	local uierrors =  {}

	local uilistbuttonsInput = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttonsInput
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "TextInput",
		id = "filterModelErrors",
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		id = "filterModelErrorsApply",
		caption = "Filter apply",
		onclick = function(item, win)
			win:refresh()
		end
	}

	uilist[#uilist+1] = {
		typ = "Label",
		caption = "Recorded warnings and errors:"
	}

	if (commonapi._info.settings.checkmods ~= nil and commonapi._info.settings.checkmods == true) then

	else
		uilist[#uilist+1] = {
			typ = "Label",
			caption = "Please enable checkmods in settings"
		}
	end


	local columnHeaders = {
		{
			caption = "Filename / Key",
			width = 100
		},
		{
			caption = "Error",
			width = 0
		},
		{
			caption = "LOD",
			width = 10
		},
		{
			caption = "",
			width = 10
		}
	}

	local searchregex
	if (self.params.filterModelErrors ~= nil) then
		searchregex = "^"..string.gsub(self.params.filterModelErrors,'[%%%-%.%+%$%^%?%*%[%]%(%)]','%%%1')
	end

	local modelErrors = checkModels.getErrors()
	local sorted = {}
	for fileName in pairs(modelErrors) do
		if (searchregex ~= nil and (string.find(fileName, searchregex) == nil) ) then
			-- skip
		else
			sorted[#sorted+1] = fileName
		end
	end
	table.sort(sorted,
		function(a, b)
			local aS = modelErrors[a].maxseverity or 0
			local bS = modelErrors[b].maxseverity or 0
			if ( aS == bS ) then
				return (a < b)
			end
			return (aS > bS)
		end
	)

	for idxs, fileName in ipairs(sorted) do
		local d = modelErrors[fileName]

		if (d.lines ~= nil) then
			uierrors[#uierrors + 1] = {
				typ = "Button",
				caption = fileName,
				column = 0,
				rownew = true,
				id = "datatoclipboard_" .. idxs,
				tooltip = _("Copy information to clipboard (requires native code)"),
				onclick = function(item, win)
					win:copyToClipboard(fileName)
				end,
			}

			for idx, line in ipairs(d.lines) do
				uierrors[#uierrors + 1] = {
					typ = "Label",
					caption = "      "..(line.metadatapath or ""),
					column = 0,
					rownew = true
				}

				uierrors[#uierrors + 1] = {
					typ = "Label",
					caption = line.failure or line.text,
					column = 1,
				}
				local lodName = ""
				if (line.lodId ~= nil) then
					if (line.lodId > 0) then
						lodName = "LOD"..(line.lodId-1)
					else
						lodName = "?"
					end
				end
				uierrors[#uierrors + 1] = {
					typ = "Label",
					caption = lodName,
					column = 2,
				}

				local warningstr = ""
				if (line.severity ~= nil) then
					warningstr = ""..line.severity
				end
				uierrors[#uierrors + 1] = {
					typ = "Label",
					caption = warningstr,
					column = 3,
				}
			end
		end
	end

	uilistscroll[#uilistscroll+1] = {
		typ = "Table",
		id = "ModellErrorTable",
		columns = #columnHeaders,
		columnHeaders = columnHeaders,
		border = true,
		uilist = uierrors,
	}


	uilist[#uilist + 1] = {
		typ = "ScrollRegion",
		id = "ModellErrorScrollRegion",
		border = false,
		uilist = uilistscroll
	}

	return uilist
end



return LoadErrorsWindow