local ScriptHelp = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.ui2.ScriptHelp")
local ui = require("commonapi2.ui2.tpfui")

ScriptHelp = ui.Window:new()

ScriptHelp.id =  "ScriptHelp"
ScriptHelp.caption = "CommonAPI2 Script help system"
ScriptHelp.uilist = {}

ScriptHelp.height = 500
ScriptHelp.width = 700
ScriptHelp.forceSize = true


function ScriptHelp.create()
	local o
	o = ScriptHelp:new()
	o.params["HelpTabs"] = 0
	o:createUIList()
	o:open()
	return o
end

function ScriptHelp:refresh()
	self:createUIList()
	self:rebuild()
end

function ScriptHelp:createUIList()
	local uilist = {}
	local uitabs = {}

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "LineEventsScriptTab",
		caption = "Line Events",
		uilist = self:createUIListLineEvents()
	}
	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "VehicleScriptTab",
		caption = "Vehicle Script",
		uilist = self:createUIListVehicleScript()
	}

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "ConstructionScriptTab",
		caption = "Construction Script",
		uilist = self:createUIListConstructionScript()
	}

	uilist[#uilist + 1] = {
		typ = "TabBar",
		id = "HelpTabs",
		uilist = uitabs
	}


	self.uilist = uilist
end

function ScriptHelp:createUIListLineEvents()
	local uilist = {}

	local lineeventstext = [[
In line events you can use for each stop your own scripts, for easier programming, commonapi provides some modules and object to manipulate the game.
These are vehiclescript and constructionscript.
If you want to use these object in the lua console, you can use:
conobj = require("commonapi2.lineextradetails.constructionscript")
con = conobj.new(<entityid>)
con:setParameter("myparameter", 2)
con:doIt() -- will rebuild object

or

vobj = require("commonapi2.lineextradetails.vehiclescript")
vs = vobj.new(<entityid>)
...

vs:doIt() -- will change transport vehicle

WARNING: The internal path of commonapi2.lineextradetails is not guaranteed and may change in the future, you have been warned!

If the first parameter is self, you have to use lua object call syntax with colon! Example: con:setParameter("myparameter", 2).
With colon syntax the self parameter will be automatically filled.
]]
	uilist[#uilist+1] = {
		typ = "Label",
		caption = lineeventstext,
		id = "lineeventstarttext",
		setSelectable = true,
	}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = "For line events there are already some stable preloaded values that can be used directly (so you won't need any lua require)"
	}
	local params = {
		{ "vs", "VehicleScriptObject", "Already created vehicle script object for the current vehicle" },
		{ "conobj", "constructionscript", "You can use conobj.new(<yourentityid>) to access other constructions" },
		{ "testmode", "bool", "If this is only a script test run (syntax check)" }
	}
	local uiparams = {}
	for paramid, paramdata in ipairs(params) do
		uiparams[#uiparams + 1] = {
			typ = "Label",
			caption = paramdata[1],
			column = 0,
			rownew = true
		}
		uiparams[#uiparams + 1] = {
			typ = "Label",
			caption = paramdata[2],
			column = 1,
		}
		uiparams[#uiparams + 1] = {
			typ = "Label",
			caption = paramdata[3],
			column = 2,
		}
	end
	uilist[#uilist+1] = {
		typ = "Table",
		id = "LineEventScriptParams",
		columns = 3,
		columnWidths = { 160, 140, 0},
		border = false,
		uilist = uiparams,
		styleclass = "scripthelp-params"
	}

	local uilistscroll = {}
	uilistscroll[#uilistscroll + 1] = {
		typ = "ScrollRegion",
		id = "LineEventHelpScrollRegion",
		border = false,
		uilist = uilist
	}

	return uilistscroll
end

function ScriptHelp:createUIListConstructionScript()
	local ok, obj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.constructionscript")
	if (ok == true) then
		return self:createUIListScriptHelp(obj, "conscript-","Construction Script")
	end
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = ""..obj
	}
	return uilist
end

function ScriptHelp:createUIListVehicleScript()
	local ok, obj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.vehiclescript")
	if (ok == true) then
		return self:createUIListScriptHelp(obj, "vsscript-","Vehicle Script")
	end
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = ""..obj
	}
	return uilist
end

function ScriptHelp:createUIListScriptHelp(obj, id, caption)
	local uilist = {}
	uilist[#uilist+1] = {
		typ = "Label",
		caption = caption
	}
	for i, d in ipairs(obj._doc) do
		local fnparams = {}
		local uiparams = {}
		if (d.params ~= nil) then
			for paramid, paramdata in ipairs(d.params) do
				fnparams[#fnparams+1] = paramdata[1]
				uiparams[#uiparams + 1] = {
					typ = "Label",
					caption = paramdata[1],
					column = 0,
					rownew = true
				}
				uiparams[#uiparams + 1] = {
					typ = "Label",
					caption = paramdata[2],
					column = 1,
				}
				uiparams[#uiparams + 1] = {
					typ = "Label",
					caption = paramdata[3],
					column = 2,
				}
			end
		end
		local returntyp = ""
		if (d.ret ~= nil) then
			returntyp = " -> "..d.ret
		end
		uilist[#uilist+1] = {
			typ = "Label",
			caption = d.name .. "("..table.concat(fnparams,", ")..")"..returntyp,
			styleclass = "scripthelp-name"
		}
		uilist[#uilist+1] = {
			typ = "Table",
			id = id..d.name,
			columns = 3,
			columnWidths = { 160, 140, 0},
			border = false,
			uilist = uiparams,
			styleclass = "scripthelp-params"
		}

		uilist[#uilist+1] = {
			typ = "Label",
			caption = (d.desc or ""),
			styleclass = "scripthelp-desc"
		}
	end
	local uilistscroll = {}
	uilistscroll[#uilistscroll + 1] = {
		typ = "ScrollRegion",
		id = id.."HelpScrollRegion",
		border = false,
		uilist = uilist
	}

	return uilistscroll
end



function ScriptHelp.createMarkdown(p)
--[[
ScriptHelp = require "commonapi2.ui2.ScriptHelp"
ScriptHelp.createMarkdown("D:/Dev/CommonAPI2/eis_os_commonapi2_doc/scriptobjects/")
--]]
	package.loaded["commonapi2.ui2.ScriptHelp"] = nil
	local ok, ScriptHelp = pcall(require,  "commonapi2.ui2.ScriptHelp")
	if (ok == true) then
		local okcreate, error = pcall(ScriptHelp.createMarkdownIn, p)
		if (okcreate == false) then
			print(error)
		end
	else
		print(ScriptHelp)
	end
end

function ScriptHelp.createMarkdownIn(p)
	local intro = ""
	local ok, obj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.vehiclescript")
	if (ok == true) then
		intro = "The vehicle script object may return itself for easy chaining of commands or sub selection (marked as new Selection)\\\n"
		intro = intro .. "In line events there is already some predefined objects: [NewLineEvents](features/NewLineEvents.md?id=vehicle-events)"
		ScriptHelp.createScriptHelpMarkdown(obj, p, "Vehicle Script", "VehicleScript", intro)
	end

	ok, obj = pcall(commonapi.rerequire, "commonapi2.lineextradetails.constructionscript")
	if (ok == true) then
		intro = ""
		ScriptHelp.createScriptHelpMarkdown(obj, p, "Construction Script", "ConstructionScript", intro)
	end
end

function ScriptHelp.createScriptHelpMarkdown(obj, p, caption, filename, intro)
	local fullfilename = p..filename..".md"
	local file, fileerr = io.open(fullfilename, "w");
	if (file == nil) then
		print(fileerr)
		return
	end
	local header = "# " .. caption .. "\n\n"
	file:write(header)
	file:write(intro)
	file:write("\n\n")
	for i, d in ipairs(obj._doc) do
		local text = ""
		local fnparams = {}
		local localparams = ""
		if (d.params ~= nil) then
			localparams = "| Name | Type | Description |\n"
			localparams = localparams .. "| --- | --- | --- |\n"
			for paramid, paramdata in ipairs(d.params) do
				fnparams[#fnparams+1] = paramdata[1]
				localparams = localparams .. "|" .. paramdata[1] .. "|" .. (paramdata[2] or "") .. "|" ..  (paramdata[3] or "") .. "|\n"
			end
			localparams = localparams .."\n\n"
		end
		local returntyp = ""
		if (d.ret ~= nil) then
			returntyp = " -> "..d.ret
		end
		local calling = ""
		if (fnparams[1] == "self") then
			table.remove(fnparams, 1)
			calling = "obj:"
		else 
			calling = "obj."
		end
		text = text .. "## "..d.name.."\n\n"
		text = text .. calling.. d.name .. "("..table.concat(fnparams,", ")..")"..returntyp.."\n\n"
		text = text .. localparams
		text = text .. string.gsub(d.desc or "", "\n", "\\\n") .. "\n\n"
		file:write(text)
	end

	file:write("")
	file:close()
end




return ScriptHelp