local conlist = {}

local _utils = commonapi.utils
local _log = _utils.logPrep("commonapi.ui2.constructionlist")
local ui = require("commonapi2.ui2.tpfui")

local listwrapper = {}


--[[
	require("commonapi2.ui2.constructionlist")
	conlist.reload()
--]]
function listwrapper:new(o)
	local o = o or {}

	for k, v in pairs(self) do
		if (o[k] == nil) then
			o[k] = _utils.deepcopy(v)
		end
	end
	return o
end


function listwrapper:setCategoryPath(categoryPath)
	self.categoryPath = categoryPath
end

function listwrapper:setName(name)
	self.name  = name
end

function listwrapper:setRepoName(name)
	self.reponame = name
end

function listwrapper:setTooltipLayout(tooltipLayout)
	self.tooltipLayout = tooltipLayout
end


local function parseDescription(desc)
	if (desc == nil) then
		return nil, nil
	end
	local groupvalue
	local fileData_groupname, fileData_groupvalue = string.match(desc, "@!([a-zA-Z0-9-_]+[a-zA-Z_])(-?%d*)")
	if (fileData_groupname ~= nil) then
		if (fileData_groupvalue ~= nil and fileData_groupvalue ~= "") then
			groupvalue = 0 - tonumber(fileData_groupvalue)
		else
			groupvalue = 0
		end
		return fileData_groupname, groupvalue
	end
end

local function parseDescriptionBlock(r, descBlock)
	if (descBlock == nil) then
		return
	end

	if (r.groupname == nil) then
		local fileData_groupname, fileData_groupvalue = parseDescription(descBlock.description)
		if (fileData_groupname ~= nil) then
			r.groupname = fileData_groupname
			r.groupvalue = fileData_groupvalue
		end
	end
	r.description = descBlock.description or nil
	r.name = descBlock.name or nil
end

local function parseCategories(r, fileData)
	if (fileData.categories ~= nil) then
		if (#fileData.categories > 0) then
			local cats = {}
			for k, v in ipairs(fileData.categories) do
				cats[k] = v
			end
			r.categories = cats
		end
	end
end

listwrapper.repoGetter = {
}

listwrapper.repoGetter.trackTypeRep = function(repo, filename)
	local r = {}
	local groupname, groupvalue = filename:match("(.-)(%d%d+)%.lua$")

	local repoid = repo.find(filename)
	if (repoid >= 0) then
		local fileData = repo.get(repoid)
		local fileData_groupname, fileData_groupvalue = parseDescription(fileData.desc)
		if (fileData_groupname ~= nil) then
			groupname = fileData_groupname
			groupvalue = fileData_groupvalue
		end
		local speedText = api.util.formatSpeed(fileData.speedLimit or fileData.speed or 0)
		r.description = fileData.desc or nil
		r.description = r.description:gsub("%%2%%", speedText)
		r.name = fileData.name or nil
		if (fileData.categories ~= nil) then
			parseCategories(r, fileData)
		end
	end
	r.groupname = groupname
	r.groupvalue = groupvalue
	r.repoid = repoid
	return r
end
listwrapper.repoGetter.streetTypeRep = listwrapper.repoGetter.trackTypeRep

listwrapper.repoGetter.constructionRep = function(repo, filename)
	local r = {}

	local tempFilename, tempId = filename:match("(.-)_(%d+)$")
	--print("listwrapper.repoGetter.constructionRep", filename, tempFilename, tempId)

	local repoid = -1
	if (tempFilename ~= nil) then
		repoid = repo.find(tempFilename)
	end

	if (repoid < 0) then
		repoid = repo.find(filename)
		tempId = nil
	end

	if (repoid >= 0) then
		local fileData = repo.get(repoid)
		local descBlock
		if (tempId ~= nil and fileData.constructionTemplates ~= nil) then
			local templateId = tonumber(tempId)
			local conTemplate = fileData.constructionTemplates[templateId+1]
			if (conTemplate ~= nil) then
				descBlock = conTemplate.desc
				r.templateId = templateId
			end
		end
		if (r.templateId == nil and fileData.description ~= nil) then
			descBlock = fileData.description
		end

		if (descBlock ~= nil) then
			parseDescriptionBlock(r, descBlock)
		end
		if (fileData.categories ~= nil) then
			parseCategories(r, fileData)
		end
		r.repoid = repoid
		return r
	end
	return nil
end

local ModelMetadataExtGet

if (commonapi._native.ModelMetadataExt ~= nil) then
	if (commonapi._native.ModelMetadataExt.isEnabled() == true) then
		ModelMetadataExtGet = commonapi._native.ModelMetadataExt.get
	end
end

listwrapper.repoGetter.modelRep = function(repo, filename)
	local r = {}

	local repoid = repo.find(filename)
	if (repoid >= 0) then
		local fileData = repo.get(repoid)

		if (ModelMetadataExtGet ~= nil) then
			local info = ModelMetadataExtGet( repoid, "uiCommonAPI" )
			if (info ~= nil) then
				r.groupname = info.groupName or nil
				r.groupvalue = tonumber(info.groupValue or 0)
			end
		end

		if (fileData.metadata ~= nil) then
			if (fileData.metadata.description ~= nil) then
				parseDescriptionBlock(r, fileData.metadata.description)
			end
			if (fileData.metadata.category ~= nil and fileData.metadata.category.categories ~= nil) then
				parseCategories(r, fileData.metadata.category)
			end
		end

		r.repoid = repoid
		return r
	end
	return nil
end

function listwrapper:setSpecialList(speciallist)
	self.speciallist = speciallist
end

function listwrapper:setConstructionCategories(constructionCategories)
	self.constructionCategories = constructionCategories
end

function listwrapper:setConstructionList(constructionList)
	local this = self
	self.listid = "commonapi.menu.construction."..self.name

	self.constructionlist = constructionList
	self.clist = commonapi._native.UIHacks.getUIListFrom(constructionList)

	local parent = constructionList:getParent()

	local oldele = api.gui.util.getById(self.listid)
	if (oldele ~= nil) then
		local oldparent = oldele:getParent()
		oldele:setId("")
		oldparent:getLayout():removeItem(oldele)
		api.gui.util.destroyLater(oldele)
	end


	self.list = api.gui.comp.List.new(false, 0, false)
	self.list:setEmitSignalOnReselect(true)
	self.list:setId(self.listid)
	parent:getLayout():addItem(self.list)

	local clistContentLayout = self.clist:getContent():getLayout()
	self.clistContentLayout = clistContentLayout

	local speciallistbykey = {}
	if (self.speciallist ~= nil) then
		for k, v in ipairs(self.speciallist) do
			speciallistbykey[v] = v
		end
	end
	self.updateVisibilityTimed = false

	local entries = {}
	local entriesbygroupmaybe = {}
	for i=0,clistContentLayout:getNumItems()-1,1 do
		local imageView = clistContentLayout:getItem(i)
		ui._bindOnce(imageView, "onVisibilityChange", "conlist"..self.name,
			function(state)
				--print("Imageview for ", this.name, " ", imageView:getId()," new state ", state)
				if (this.updateVisibilityTimed == true) then
					return
				end
				imageView:invokeLater(
					function()
						this.updateVisibilityTimed = false
						this:updateVisibility()
					end
				)
				this.updateVisibilityTimed = true
			end
		)

		local isVisibleFn = function(that)
			return imageView:isVisible()
		end

		local imageViewImage = commonapi._native.UIHacks.getImageViewImage(imageView)
		local filename = imageView:getId():gsub("^menu%.construction%.%a+%.[a-zA-Z-]+%.item%.", "")
		--print("Item: "..i.." "..filename.." "..imageView:getId() .. " image=" ..imageViewImage.." visible=", isVisibleFn() )

		local rData
		if (filename:match("%.mdl$") ~= nil) then
			rData = listwrapper.repoGetter.modelRep(api.res.modelRep, filename)
		end
		if (self.reponame ~= nil and rData == nil or rData.repoid < 0) then
			rData = listwrapper.repoGetter[self.reponame](api.res[self.reponame], filename)
		end

		if (rData == nil) then
			rData = {}
		end

		local groupname = rData.groupname or nil
		local groupvalue = rData.groupvalue or nil
--[[
	require("commonapi2.ui2.constructionlist")
	conlist = commonapi.rerequire("commonapi2.ui2.constructionlist")
	conlist.reload()
--]]

		local function padnum(d) 
			local dec, n = string.match(d, "(%.?)0*(.+)")
			return #dec > 0 and ("%.12f"):format(d) or ("%s%03d%s"):format(dec, #n, n)
		end

		local entry = {
			filename = filename,
			filename_sort = filename:gsub("%.?%d+",padnum),
			image = imageViewImage,
			doit = self:createUIListCallableEntry(self.clist, i),
			clistId = i,
			isVisible = isVisibleFn,
			repoid = rData.repoid or -1,
			repo = rData.repo or nil,
			name = rData.name or nil,
			description = rData.description or nil,
			categories = rData.categories or nil,
			isSpecial = false
		}
		if (speciallistbykey[imageViewImage]) then
			entry.isSpecial = true
		end

		if (groupname ~= nil and groupvalue ~= nil) then
			entry.groupname = groupname
			entry.groupvalue = tonumber(groupvalue) or 0
			if (entriesbygroupmaybe[groupname] == nil) then
				entriesbygroupmaybe[groupname] = {}
			end
			entriesbygroupmaybe[groupname][ #entriesbygroupmaybe[groupname] + 1] = entry
		end
		entries[#entries+1] = entry
		--local newButton = api.gui.comp.Button.new(newImageView, true)
		--self.list:addItem(newImageView)
	end


	local sortfn = function(a, b)
		-- if (a.isSpecial == true and b.isSpecial == true) then
		--	return false
		-- end
		if (a.isSpecial == true and b.isSpecial == false) then
			return true
		end
		if (a.isSpecial == false and b.isSpecial == true) then
			return false
		end
		return a.filename_sort:upper() < b.filename_sort:upper()
	end

	local sortfn2 = function(a, b)
		return a.groupvalue < b.groupvalue
	end

	local entriesbygroup = {}

	for k, v in pairs(entriesbygroupmaybe) do
		if (#v > 1) then
			entriesbygroup[k] = v
			--table.sort(	entriesbygroup[k], sortfn2)
		else
			v[1].groupname = nil
			v[1].groupvalue = nil
		end
	end
	entriesbygroupmaybe = {}

	table.sort(entries, sortfn)


	for k, v in pairs(entriesbygroup) do
		table.sort(v, sortfn2)
	end

	local uilist = {}
	self.uilist = uilist

	local groupsused = {}
	for k, entry in pairs(entries) do
		if (entry.groupname) then
			local groupname = entry.groupname
			if (groupsused[groupname] ~= true) then
				uilist[#uilist+1] = self:_newGroupEntry(groupname, entriesbygroup[groupname])
				groupsused[groupname] = true
			end
		else
			uilist[#uilist+1] = self:_newEntry(entry)
		end
	end

	for k, e in pairs(uilist) do
		self.list:addItem(e.ele)
	end

	self.list:onSelect(
		function(idx)
			if (idx > #this.uilist) then
				return
			end
			this.uilist[idx+1].doit(this.uilist[idx+1])
		end
	)

	self.list:onHover(
		function(idx)
			if (idx < 0 or idx > #this.uilist) then
				local ok, err = pcall(this.showTooltip, this, nil)
				if (ok == false) then
					commonapi.dmp(err)
				end
			else
				local ok, err = pcall(this.showTooltip, this, this.uilist[idx+1])
				if (ok == false) then
					commonapi.dmp(err)
				end
			end
		end
	)
	--self.list = api.gui.comp.List.new()
end

function listwrapper:updateVisibility()
	--print("listwrapper:updateVisibility")
	for k, e in pairs(self.uilist) do
		e.ele:setVisible(e:isVisible(), false)
	end
end


function listwrapper:createUIListCallableEntry(list, relativeid)
	local this = self
	local fn = function(entry)
		list:select(relativeid, true)
		this:constructionCategoriesHighlightByEntry(entry)
	end
	return fn
end

function listwrapper:_newEntry(entry)
	local layout = api.gui.layout.BoxLayout.new("VERTICAL")
	entry.ele = api.gui.comp.Component.new("commonapi-ConlistButton")
	entry.ele:setLayout(layout)
	local imageView = api.gui.comp.ImageView.new(entry.image)
	imageView:addStyleClass("con-entry-icon")
	imageView:addStyleClass("con-entry-single")

	layout:addItem(imageView)
	entry.ele:setVisible(entry:isVisible(), false)
	return entry
end

--[[
	require("commonapi2.ui2.constructionlist")
	conlist.reload()
--]]

function listwrapper:createPopupMenu(parent, entries, curentry, subselector)
	local this = self
	local popupList = api.gui.comp.List.new(false, 1, false)
	popupList:setName("commonapi-ConListPopup")
	local screenSize = ui._getRoot():getContentRect()

	local maxsize = api.gui.util.Size.new( screenSize.w - 100, screenSize.h - 250 )

	local scale = ui.getUIScale()
	maxsize.w = math.floor(maxsize.w / scale)
	maxsize.h = math.floor(maxsize.h / scale)

	popupList:setMaximumSize(maxsize)

	local subselectorZIndex = function(idx)
		local ok, err = pcall(subselector, entries[idx+1])
		if (ok == false) then
			commonapi.dmp(err)
		end
	end
	for k, v in ipairs(entries) do
		local imageView = api.gui.comp.ImageView.new(v.image)
		imageView:addStyleClass("con-entry-icon")
		imageView:addStyleClass("con-entry-single")
		imageView:setVisible(v:isVisible(), false)
		popupList:addItem(imageView)
		if (v == curentry) then
			popupList:select(k-1, false)
			popupList:invokeLater(function()
				popupList:ensureVisible(imageView)
			end)
		end
	end
	popupList:onSelect(subselectorZIndex)
	popupList:setEmitSignalOnReselect(true)

	popupList:onHover(
		function(idx)
			if (idx < 0 or idx > #entries) then
				local ok, err = pcall(this.showTooltip, this, nil)
				if (ok == false) then
					commonapi.dmp(err)
				end
			else
				local ok, err = pcall(this.showTooltip, this, entries[idx+1])
				if (ok == false) then
					commonapi.dmp(err)
				end
			end
		end
	)

	commonapi._native.UIHacks.showPopupMenu(parent, popupList, parent:getContentRect(), 4, 0.0)
end



function listwrapper:_newGroupEntry(groupname, entries)
	local groupentry = {
		isGroup = true,
		entries = entries,
		curentry = entries[1]
	}
	local layout = api.gui.layout.BoxLayout.new("VERTICAL")
	groupentry.ele = api.gui.comp.Component.new("commonapi-ConlistButton")
	groupentry.ele:setLayout(layout)
	groupentry.imageView = api.gui.comp.ImageView.new(groupentry.curentry.image)
	groupentry.imageView:addStyleClass("con-entry-icon")
	groupentry.isVisible = function(that)
		for k, e in pairs(that.entries) do
			if (e:isVisible()) then
				return true
			end
		end
		return false
	end

	local changeCurEntry = function(entry)
		groupentry.curentry = entry
		groupentry.imageView:setImage(groupentry.curentry.image, true)

		groupentry.name = groupentry.curentry.name or nil
		groupentry.description = groupentry.curentry.description or nil
		groupentry.filename = groupentry.curentry.filename or nil
		groupentry.categories = groupentry.curentry.categories or nil
		groupentry.repoid = groupentry.curentry.repoid or nil
	end
	changeCurEntry( entries[1] )

	groupentry.ele:setVisible(groupentry:isVisible(), false)

	local this = self
	local subselector = function(entry)
		changeCurEntry( entry )
		for k, e in pairs(this.uilist) do
			if (e == groupentry) then
				this.list:select(k-1, true)
			end
		end
		commonapi._native.UIHacks.removeAllPopupMenus()
	end

	local subMenuImage = api.gui.comp.ImageView.new("ui/small_button_arrow_up.tga")
	groupentry.subMenuButton = api.gui.comp.Button.new(subMenuImage, true)
	groupentry.subMenuButton:addStyleClass("con-entry-submenu")
	groupentry.subMenuButton:setGravity(0.5, 0)

	groupentry.subMenuButton:onClick(function()
		this:createPopupMenu(groupentry.subMenuButton, groupentry.entries, groupentry.curentry, subselector)
	end)
	layout:addItem(groupentry.subMenuButton)
	layout:addItem(groupentry.imageView)



	groupentry.doit = function()
		if (groupentry.curentry ~= nil) then
			groupentry.curentry.doit(groupentry.curentry)
		end
	end

	return groupentry
end

function listwrapper:showTooltip(entry)
	if (self.tooltipLayout == nil) then
		return
	end
--	if (entry == nil) then
		--self.tooltipLayout:deleteAll()
		local numitems = self.tooltipLayout:getNumItems()
		if (numitems > 0) then
			for i = numitems, 1, -1 do
				local tmp = self.tooltipLayout:getItem(i-1)
				self.tooltipLayout:removeItem(tmp)
				api.gui.util.destroyLater(tmp)
			end
		end
	if (entry == nil) then
		return
	end
	local info_layout = api.gui.layout.BoxLayout.new("VERTICAL")
	local info_ele = api.gui.comp.Component.new("ConstructionList::Info")
	info_ele:setLayout(info_layout)
	info_ele:addStyleClass("info-popup")

	if (entry.name ~= nil) then
		local name_ele = api.gui.comp.TextView.new(entry.name or "")
		name_ele:addStyleClass("title")
		info_layout:addItem(name_ele)
	end

	if (entry.description ~= nil) then
		local desc_ele = api.gui.comp.TextView.new("")
		commonapi._native.UIHacks.setTextViewMaxSize(desc_ele, 360)
		desc_ele:setText(entry.description or "")
		desc_ele:addStyleClass("description")
		info_layout:addItem(desc_ele)
	end


	if (entry.filename ~= nil) then
		local filename_ele = api.gui.comp.TextView.new("")
		commonapi._native.UIHacks.setTextViewMaxSize(filename_ele, 360)
		local filename = entry.filename or ""
		if (entry.repoid ~= nil) then
			filename = filename .. " ["..entry.repoid.."]"
		end
		filename_ele:setText(filename)
		filename_ele:addStyleClass("description")
		info_layout:addItem(filename_ele)
	end


	if (entry.categories ~= nil) then
		local categories_ele = api.gui.comp.TextView.new("")
		local categories_str = table.concat(entry.categories, ", ")
		commonapi._native.UIHacks.setTextViewMaxSize(categories_ele, 360)
		categories_ele:setText(categories_str or "")
		categories_ele:addStyleClass("description")
		info_layout:addItem(categories_ele)
	end
--[[
	if (entry.groupvalue ~= nil) then
		local groupvalue_ele = api.gui.comp.TextView.new("")
		--commonapi._native.UIHacks.setTextViewMaxSize(filename_ele, 360)
		groupvalue_ele:setText("groupvalue: "..tostring(entry.groupvalue))
		groupvalue_ele:addStyleClass("description")
		info_layout:addItem(groupvalue_ele)
	end
--]]
	self.tooltipLayout:addItem(info_ele, 1, 1)
end

function listwrapper:constructionCategoriesHighlightByEntry(entry)
	if (self.constructionCategories == nil) then
		return
	end

	local catmap = {}
	if (entry ~= nil and entry.categories ~= nil) then
		for k, cat in ipairs(entry.categories) do
			local catid = "menu.construction."..self.categoryPath ..".item."..cat
			catmap[catid] = true;
		end
	end


	local catLayout = self.constructionCategories:getLayout()
	if (catLayout ~= nil) then
		local catNumitems = catLayout:getNumItems()
		if (catNumitems > 0) then
			for i = catNumitems, 1, -1 do
				local tmp = catLayout:getItem(i-1)
				local catid = tmp:getId()
				if (catmap[catid] == true) then
					tmp:addStyleClass("commonapi-cathighlight")
					tmp:removeStyleClass("commonapi-catnohighlight")
				else
					tmp:addStyleClass("commonapi-catnohighlight")
					tmp:removeStyleClass("commonapi-cathighlight")
				end
			end
		end
	end
end

function listwrapper:constructionCategoriesPrepareHighlight(enable)
	if (self.constructionCategories == nil) then
		return
	end
	local catLayout = self.constructionCategories:getLayout()
	if (catLayout ~= nil) then
		local catNumitems = catLayout:getNumItems()
		if (catNumitems > 0) then
			for i = catNumitems, 1, -1 do
				local tmp = catLayout:getItem(i-1)
				if (enable == true) then
					tmp:removeStyleClass("commonapi-cathighlight")
					tmp:addStyleClass("commonapi-catnohighlight")
				else
					tmp:removeStyleClass("commonapi-catnohighlight")
					tmp:removeStyleClass("commonapi-cathighlight")
				end
			end
		end
	end
end

function conlist.reload()
	package.loaded["commonapi2.ui2.tpfui"] = nil
	package.loaded["commonapi2.ui2.constructionlist"] = nil
	print("Reloading...")
	local myconlist = require("commonapi2.ui2.constructionlist")
	local ok, err = pcall(myconlist.init)
	if (ok == false) then
		commonapi.dmp(err)
	end
end

function conlist.init()
	conlist.patchModelGroups()
	conlist.getImportantWindowElements()

end


function conlist.createNewList(tabele, catpath, name, reponame, tooltipEle, speciallist)
	local constructionList = ui._findByName(tabele, "ConstructionList", 1)
	local constructionCategories = ui._findByName(tabele, "ConstructionMenuTab::Categories", 5)
	constructionList:setVisible(true, false)
	local conlistobj = listwrapper:new()
	conlistobj:setCategoryPath(catpath)
	conlistobj:setName(name)
	conlistobj:setRepoName(reponame)
	conlistobj:setSpecialList(speciallist)
	conlistobj:setConstructionList(constructionList)
	conlistobj:setConstructionCategories(constructionCategories)
	conlist[name] = conlistobj

	if (tooltipEle ~= nil) then
		conlistobj:setTooltipLayout(tooltipEle:getLayout())
	end


	local ConstructionMenuTabLayout = tabele:getLayout()
	local toggleId = "commonapi.menu.construction.toggle."..name

	conlistobj.ConstructionMenuTabLayout = ConstructionMenuTabLayout

	local oldele = api.gui.util.getById(toggleId)
	if (oldele ~= nil) then
		local oldparent = oldele:getParent()
		oldele:setId(toggleId.."_old")
		oldparent:getLayout():removeItem(oldele)
		api.gui.util.destroyLater(oldele)
	end

	local cstyle = {"ui/onoff0.tga", "ui/onoff1.tga"}
	local toggle = api.gui.comp.CheckBox.new(_("Grouping"), cstyle[1], cstyle[2])
	toggle:setId(toggleId)
	toggle:onToggle(
		function(newbool)
			--constructionList:setVisible(not newbool, false)
			local s
			if (newbool) then
				s = api.gui.util.Size.new(1, 1)
			else
				s = api.gui.util.Size.new(-1,-1)
			end
			constructionList:setMaximumSize(s)
			conlistobj.list:setVisible(newbool, false)
			conlistobj:constructionCategoriesPrepareHighlight(newbool)
		end
	)
	--ConstructionMenuTabLayout:addItem(toggle)
	ConstructionMenuTabLayout:insertItem(toggle, 1)
	toggle:setSelected(true, true)

end

--[[
	require("commonapi2.ui2.constructionlist")
	conlist.reload()
--]]


function conlist.getImportantWindowElements()
	local roadmenu, railmenu, terrainmenu

	roadmenu = api.gui.util.getById("menu.construction.roadmenu")
	railmenu = api.gui.util.getById("menu.construction.railmenu")
	terrainmenu = api.gui.util.getById("menu.construction.terrainmenu")

	local roadtabs = ui._findByNameAll(roadmenu, "ConstructionMenuTab", 3)
	local railtabs = ui._findByNameAll(railmenu, "ConstructionMenuTab", 3)
	local terraintabs = ui._findByNameAll(terrainmenu, "ConstructionMenuTab", 3)

	local roadmenu_tooltip = ui._findByName(roadmenu, "ConstructionMenu::Info", 3)
	local railmenu_tooltip = ui._findByName(railmenu, "ConstructionMenu::Info", 3)
	local terrainmenu_tooltip = ui._findByName(terrainmenu, "ConstructionMenu::Info", 3)

	conlist.roadmenu = roadmenu
	conlist.railmenu = railmenu
	conlist.terrainmenu = terrainmenu

	conlist.createNewList(roadtabs[1], "road.streets", "road", "streetTypeRep", roadmenu_tooltip)
	conlist.createNewList(roadtabs[2], "road.road-buildings", "road-buildings", "constructionRep", roadmenu_tooltip)

	conlist.createNewList(railtabs[1], "rail.tracks", "rail", "trackTypeRep", railmenu_tooltip)
	conlist.createNewList(railtabs[2], "rail.rail-buildings", "rail-buildings", "constructionRep", railmenu_tooltip)

	local specialTools = {
		"ui/button/large/terrain_reset_material.tga",
		"ui/construction/asset/mixed_tree.tga",
		"ui/construction/asset/mixed_rock.tga"
	}
	conlist.createNewList(terraintabs[3], "terrain.assets", "terrain-assets", "constructionRep", terrainmenu_tooltip, specialTools)
end


function conlist.patchModelGroups()
	local groupFix = {
		["asset/rocks/rock_granite_01.mdl"] = "rock_granite",
		["asset/rocks/rock_granite_02.mdl"] = "rock_granite",
		["asset/rocks/rock_granite_03.mdl"] = "rock_granite",
		["asset/rocks/rock_granite_04.mdl"] = "rock_granite",

		["asset/rocks/rock_cracked_01.mdl"] = "rock_cracked",
		["asset/rocks/rock_cracked_02.mdl"] = "rock_cracked",
		["asset/rocks/rock_cracked_03.mdl"] = "rock_cracked",
		["asset/rocks/rock_cracked_04.mdl"] = "rock_cracked",

		["asset/rocks/rock_sandstone_01.mdl"] = "rock_sandstone",
		["asset/rocks/rock_sandstone_02.mdl"] = "rock_sandstone",
		["asset/rocks/rock_sandstone_03.mdl"] = "rock_sandstone"
	}

	if (commonapi._native.ModelMetadataExt.isEnabled() ~= true) then
		return
	end

	for fileName, groupname in pairs(groupFix) do
		local modelId = api.res.modelRep.find(fileName)

		local uiCommonAPI = {
			groupName = groupname
		}
		commonapi._native.ModelMetadataExt.set( modelId, "uiCommonAPI", uiCommonAPI )
--[[
		local model = api.commonapiResRW.modelRep.get(modelId)
		local desc = model.metadata.description.description
		desc = string.gsub(desc, "@!([a-zA-Z0-9-_]+[a-zA-Z_])(-?%d?)", "")
		desc = desc.." @!"..groupname
		model.metadata.description.description = desc
		--commonapi._native.ModelMetadataExt.set( modelId, "uiCommonAPI", uiCommonAPI )
--]]
	end
end

--rawset(_G, "conlist", conlist)
return conlist