local InspectWindow = {}

local ui = require("commonapi2.ui2.tpfui")
local inspectlua = require("commonapi2.extlib.inspect")

local zoneutil = require "mission.zone"

local commonapimath = require("commonapi2._math")
local svec3 = commonapimath.svec3
local vec3 = commonapimath.vec3

require("serialize")
InspectWindow = ui.Window:new()

InspectWindow.id = "InspectWindow"
InspectWindow.caption = "Inspector"
InspectWindow.uilist = {}

InspectWindow.height = 500
InspectWindow.width = 500
InspectWindow._zoneid = "commonapi2_InspectWindow_zone"
InspectWindow._zoneSizes = {
	5,
	10,
	20,
	40,
	50
}

InspectWindow._zoneSizeDefault = 0

local secondaryui = require "commonapi2.ui2.secondaryui"

local hasParcelFace = false
if (commonapi._native and commonapi._native.BuildingBuilder) then
	commonapi._native.BuildingBuilder.installParcelFace()
	hasParcelFace = true
end


local function dmpModels()
	local modelIds = {}
	local fnX = function(inst)
		for x, d in pairs(inst) do
			local i = ""..d.modelId
			if (modelIds[i] == nil) then
				modelIds[i] = api.res.modelRep.getName(d.modelId)
			end
		end
	end
	local fn = function(id)
		local c = api.engine.getComponent(id, api.type.ComponentType.MODEL_INSTANCE_LIST)
		fnX(c.thinInstances)
		fnX(c.fatInstances)
	end
	api.engine.forEachEntityWithComponent(fn, api.type.ComponentType.MODEL_INSTANCE_LIST)

	local file, fileerr = io.open("dump-modelids.txt", "w");
	if (file ~= nil) then
		for x, name in pairs(modelIds) do
			file:write(name.."\n")
		end
		file:close()
	end
end

local function EventTraceOutputCallback(idsource, id, name, param)
	--EventTraceOutput
	local eleid = "XCoTPF2.InspectWindow.EventTraceScrollRegion"
	local ele = api.gui.util.getById(eleid)
	if (ele ~= nil) then
		local layout = ele:getContent():getLayout()
		if (layout) then
			local itemtext = idsource .. " " .. id .. "  " .. name .. "\n" .. inspectlua(param, {metatables = 1})
			local item = api.gui.comp.TextView.new(itemtext)
			layout:addItem(item, false)

			if (layout:getNumItems() > 10) then
				local tempitem = layout:getItem(0)
				layout:removeItem(tempitem)
				tempitem:destroy()
			end
		--	layout:ensureVisible(item)
		end
	else
		secondaryui.traceGUICallback = nil
	end
end

local function EventTraceOutputStart(traceGameEvents)
	if (traceGameEvents == true) then
		secondaryui.traceGUICallback = nil
		local cmd = api.cmd.make.sendScriptEvent("inspect.lua", "__commonapi2__", "eventTrace", {
			enable = true
		})
		api.cmd.sendCommand(cmd)
		return
	end
	secondaryui.traceGUICallback = EventTraceOutputCallback
end

local function EventTraceOutputStop()
	secondaryui.traceGUICallback = nil
	local cmd = api.cmd.make.sendScriptEvent("inspect.lua", "__commonapi2__", "eventTrace", {
		enable = false
	})
	api.cmd.sendCommand(cmd)
end

function InspectWindow.create()
	local o
	o = InspectWindow:new()
	o:createUIList()
	o:open()
	o:setPositionCenter()

	return o
end

function InspectWindow:refresh()
	self:createUIList()
	self:rebuild()
end


function InspectWindow:refreshKeepPos()
	local eleid = self._ugidprefix .. "ScrollRegion"
	local ele = api.gui.util.getById(eleid)
	local scrollOffset = ele:getScrollOffset()
	local y = scrollOffset.y
	self:refresh()
	--commonapi.dmp(scrollOffset)

	local fn = function()
		--print("invokeLater fn")
		--commonapi.dmp(scrollOffset)
		local fnele = api.gui.util.getById(eleid)
		local fnscrollOffset = ele:getScrollOffset()
		fnscrollOffset.y = y
		fnele:setScrollOffset(fnscrollOffset)
	end
	local _ugwindow = self._ugwindow
	local fn2 = function()
		_ugwindow:invokeLater(fn)
	end
	self._ugwindow:invokeLater(fn2)
end

function InspectWindow:onclose()
	if (game and game.interface and game.interface.setZone) then
		game.interface.setZone(self._zoneid)
	end
	self:clearMakers()
	--print("InspectWindow:onclose called")
end

function InspectWindow:getComboStrings(values)
	local strings = {}
	for idx, value in ipairs(values) do
		strings[idx] = tostring(value)
	end
	if (values[1] == 0) then
		strings[1] = self._("Default")
	end
	return strings
end

function InspectWindow:getComboValue(name, values, defaultIndex)
	local comboIndex = self.params[name]
	if (comboIndex == nil) then
		if (defaultIndex ~= nil) then
			comboIndex = defaultIndex
		else
			comboIndex = 0
		end
	end

	local value = values[comboIndex + 1] or 0
	return value
end

function InspectWindow:createUIList()
	local uilist = {}
	local uitabs = {}
	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "EntityTab",
		caption = "Entity",
		uilist = self:createUIListTabEntity()
	}

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "InspectRadiusTab",
		caption = "Entities",
		uilist = self:createUIListTabRadius()
	}
-- [[
	if (commonapi and commonapi._native and commonapi._native.UIHacks) then
		uitabs[#uitabs + 1] = {
			typ = "TabItem",
			id = "RenderSettingsTab",
			caption = "Render Settings",
			uilist = self:createUIListTabRenderSettings()
		}
	end
--]]

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "UITab",
		caption = "UI",
		uilist = self:createUIListTabUI()
	}

	uitabs[#uitabs + 1] = {
		typ = "TabItem",
		id = "EventTab",
		caption = "Events",
		uilist = self:createUIListTabEvents()
	}

	uilist[#uilist + 1] = {
		typ = "TabBar",
		id = "InspectorTabs",
		uilist = uitabs
	}
	self.uilist = uilist
end

function InspectWindow:createUIListTabRenderSettings()
	local uilist = {}

	local gameui = api.gui.util.getGameUI()
	local settings = commonapi._native.UIHacks.getRenderSettings(gameui)

	uilist[#uilist + 1] = {
		typ = "Label",
		caption = "Render settings",
	}
	local settingsKeys = {
		"freezeRenderer",
		"showWireframe",
		"showWireframeStreets",
		"renderTerrain",
		"hideParcels",
		"renderStatic",
		"renderModelsOpaque",
		"renderModelsTransparent",
		"renderParticles",
		"showEdgeSteps",
		"showLanes",
		"a11",
		"showBoundingBoxes",
	--	"showSomethingBoxes",
		"grassPrepass",
		"terrainPrepass",
		"terrainDisableFilter",
		"terrainEnableTessellation",
	}
	for idx, name in pairs(settingsKeys) do
		if (name ~= "new") then
			local id = "rendersettings_"..name
			self.params[id] = settings[name]

			uilist[#uilist + 1] = {
				typ = "Checkbox",
				id = id,
				caption = name,
				onclick = function(item, win)
					local cursettings = commonapi._native.UIHacks.getRenderSettings( api.gui.util.getGameUI() )
					cursettings[name] = win.params[item.id]
					win:refresh()
				end
			}

		end
	end
	return uilist
end

function InspectWindow:createUIListTabRadiusZone()
	local radius = self:getComboValue("ZoneSize", self._zoneSizes, self._zoneSizeDefault)
	local cameraPos = game.gui.getCamera()
	-- x, y, z, turn, tilt
	local pos = {
		cameraPos[1],
		cameraPos[2]
	}

	local num = 32
	if (radius < 20) then
		num = 8
	elseif radius < 100 then
		num = 12
	elseif radius < 160 then
		num = math.floor(radius / 5)
	end

	local zonedata = {
		polygon = zoneutil.makeCircleZone(pos, radius, num),
		draw = true,
		drawColor = {.8, .8, .8, .7}
	}
	game.interface.setZone(self._zoneid, zonedata)
	self._zonedata = {
		pos = pos,
		radius = radius
	}
end
function InspectWindow:createUIListTabRadius()
	local uilist = {}

	local uilistbuttonsZone = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttonsZone
	}

	uilistbuttonsZone[#uilistbuttonsZone + 1] = {
		typ = "Button",
		id = "GetZoneItems",
		caption = self._("Get Entities"),
		onclick = function(item, win)
			win:createUIListTabRadiusZone()
			win:refresh()
		end
	}

	uilistbuttonsZone[#uilistbuttonsZone + 1] = {
		typ = "ComboBox",
		id = "ZoneSize",
		values = self:getComboStrings(self._zoneSizes),
		defaultIndex = self._zoneSizeDefault,
		onclick = function(item, win)
			win:createUIListTabRadiusZone()
			win:refresh()
		end
	}

--[[
	uilistbuttonsZone[#uilistbuttonsZone + 1] = {
		typ = "Button",
		id = "GetModelsDmp",
		caption = self._("Dump Models"),
		onclick = function(item, win)
			dmpModels()
		end
	}
--]]

	local uiTableData = {}

	local headerColumns = {
		{
			caption = "EntityId",
			width = 50
		},
		{
			caption = _("Type"),
			width = 50
		},
		{
			caption = "Name"
		}
	}
	local TableHeader = {
		typ = "TableHeader",
		items = headerColumns,
		onchange = function(item, win)
			win.cursorting = item.cursorting
			win:refresh()
		end
	}
	uiTableData[#uiTableData + 1] = TableHeader

	uilist[#uilist + 1] = {
		typ = "Table",
		id = "EntitiesInZoneTable",
		columns = #headerColumns,
		border = false,
		uilist = uiTableData,
		size = {
			0,
			-100
		},
		flags = {
			Sortable = true,
			MultiSortable = true,
			ScrollFreezeTopRow = true
		},
		onclick = function(item, win)
			local index = win.params["EntitiesInZoneTable"]
			if (index >= 0 and self._zoneEntities) then
				local entityid = self._zoneEntities[index + 1]
				if (entityid) then
					win.params.EntityId = tostring(entityid)
				end
			end
			win:refresh()
		end
	}

	if (self._zonedata) then
		local entities = game.interface.getEntities(self._zonedata, {includeData = false})
		self._zoneEntities = entities
		for _, entityid in pairs(entities) do
			self:createUIListTabRadius_CreateRow(uiTableData, entityid)
		end
	end
	return uilist
end

local ComponentTypes = api.type.ComponentType
local typeMap = {
	{ComponentTypes.BASE_EDGE_STREET, "STREET"},
	{ComponentTypes.BASE_EDGE_TRACK, "TRACK"},
	{ComponentTypes.MODEL_PERSON, "PERSON"},
	{ComponentTypes.CONSTRUCTION, "CONSTRUCTION"},
	{ComponentTypes.ASSET_GROUP, "ASSET"},
	{ComponentTypes.STATION, "STATION"},
	{ComponentTypes.STATION_GROUP, "STATION_GROUP"},
	{ComponentTypes.TOWN_BUILDING, "TOWN_BUILDING"},
	{ComponentTypes.VEHICLE_DEPOT, "DEPOT"},
	{ComponentTypes.SIGNAL_LIST, "SIGNAL"},
	{ComponentTypes.SIM_BUILDING, "SIM_BUILDING"},
	{ComponentTypes.BASE_NODE, "BASE_NODE"},
	{ComponentTypes.TRANSPORT_VEHICLE, "TRANSPORT_VEHICLE"},
	{ComponentTypes.RAILROAD_CROSSING, "RAILROAD_CROSSING"}
}
local function getEntityType(entityId)
	for _, typeMapEntry in ipairs(typeMap) do
		local componentData = api.engine.getComponent(entityId, typeMapEntry[1])
		if (componentData) then
			return typeMapEntry[2]
		end
	end
	return nil
end

function InspectWindow:createUIListTabRadius_CreateRow(uilist, entityId)
	local uientryId = {
		typ = "Label",
		caption = tostring(entityId),
		rownew = true,
		column = 0
	}
	uilist[#uilist + 1] = uientryId

	local nameColumn = ""
	if (api.engine.entityExists(entityId)) then
		local nameComponent = api.engine.getComponent(entityId, api.type.ComponentType.NAME)
		if (nameComponent) then
			nameColumn = nameComponent.name
		end

		local type = getEntityType(entityId)
		if (type) then
			uilist[#uilist + 1] = {
				typ = "Label",
				caption = type,
				column = 1
			}
		end
	end

	uilist[#uilist + 1] = {
		typ = "Label",
		caption = nameColumn,
		column = 2
	}
end

function InspectWindow:addTextToConsole(newData)
	local consoleWin = api.gui.util.getById("inGameConsole")
	if (consoleWin == nil) then
		return
	end
	local consoleInput = ui._findByName(consoleWin, "InGameConsole::TextInput")
	local curData = consoleInput:getText()

	consoleInput:setText(curData..newData, false)
	consoleInput:setFocus()
end

function InspectWindow:showComponentInConsole(entityId, typeName)
	local newData
	if (typeName == "SHAPE_LIST(_ATTRIBUTES)") then
		newData = " commonapi._native.StreetTypes.getShapeAttributes( api.engine.getComponent("..entityId..", api.type.ComponentType.SHAPE_LIST).shapes[1] )"
	else
		newData = " api.engine.getComponent("..entityId..", api.type.ComponentType."..typeName..")"
	end
	self:addTextToConsole(newData)
end

local getRefsFn = {
	STATION = function(entityId, component)
		return {  ["Construction Entity"] = { api.engine.system.streetConnectorSystem.getConstructionEntityForStation(entityId) } }
	end,
	STATION_GROUP = function(entityId, component)
		return {  ["Station Entities:"] = component.stations }
	end,
	TOWN_BUILDING = function(entityId, component)
		return {
			["Construction Entity"] = { api.engine.system.streetConnectorSystem.getConstructionEntityForTownBuilding(entityId) },
			["Town"] = { component.town },
			["Parcels"] = component.parcels
		}
	end,
	PARCEL = function(entityId, component)
		return {
			["Street segment"] = { component.streetSegment },
			["Town Building"] = { api.engine.system.townBuildingSystem.getBuilding(entityId) },
		}
	end,

	SIM_BUILDING  = function(entityId, component)
		return { ["Construction Entity"] = { api.engine.system.streetConnectorSystem.getConstructionEntityForSimBuilding(entityId) } }
	end,
	VEHICLE_DEPOT = function(entityId, component)
		return { ["Construction Entity"] = { api.engine.system.streetConnectorSystem.getConstructionEntityForDepot(entityId) } }
	end,
	CONSTRUCTION = function(entityId, component)
		return {
			["Stations"] = component.stations,
			["Depots"] = component.depots,
			["SimBuildings"] = component.simBuildings,
			["TownBuildings"] = component.townBuildings,
		}
	end,
	TRAIN = function(entityId, component)
		return {
			["Vehicles"] = component.vehicles,
		}
	end,
	BASE_EDGE = function(entityId, component)
		local objects = {}

		for i, objectref in ipairs(component.objects) do
			objects[#objects+1] = objectref[1]
		end
		return {
			["Nodes"] = { component.node0, component.node1 },
			["Objects"] = objects,
			["Parallel Strips"] = api.engine.system.baseParallelStripSystem.getStrips(entityId),
		}
	end,
	BASE_EDGE_STREET = function(entityId, component)
		local parcels = api.engine.system.parcelSystem.getSegment2ParcelData()
		local leftParcels = {}
		local rightParcels = {}
		if (parcels[entityId] ~= nil) then
			local parcelData = parcels[entityId]
			for i, id in ipairs(parcelData.leftEntities) do
				leftParcels[#leftParcels+1] = id
			end
			for i, id in ipairs(parcelData.rightParcels) do
				rightParcels[#rightParcels+1] = id
			end

			return {
				["leftParcels"] = leftParcels,
				["rightParcels"] = rightParcels,
			}
		else
			return {}
		end
	end,

	BASE_NODE = function(entityId, component)
		local n2streets = api.engine.system.streetSystem.getNode2StreetEdgeMap()
		local streetIds = {}
		local entity2streets = n2streets[entityId] or {}

		for idx, streetId in ipairs(entity2streets) do
			streetIds[#streetIds+1] = streetId
		end

		local n2tracks = api.engine.system.streetSystem.getNode2TrackEdgeMap()
		local trackIds = {}
		local entity2tracks = n2tracks[entityId] or {}

		for idx, trackId in ipairs(entity2tracks) do
			trackIds[#trackIds+1] = trackId
		end
		return {
			["Streets"] = streetIds,
			["Tracks"] = trackIds,
		}
	end,
}


function InspectWindow:zoneMakerChange(id, pos, radius, color, polygon)
	if (self._zonemarkers == nil) then
		self._zonemarkers = {}
	end

	local olddata = self._zonemarkers[id]
	if (olddata ~= nil) then
		if (olddata.pos[1] == pos[1] and olddata.pos[2] == pos[2]) then
			return
		end
	end
	if (color == nil) then
		color = {.8, .8, .8, .7}
	end
	local newdata = {
		pos = {
			pos[1], pos[2]
		},
		radius = radius,
		color = color
	}
	self._zonemarkers[id] = newdata

	if (polygon == nil) then
		polygon = zoneutil.makeCircleZone(newdata.pos, newdata.radius, 16)
	end
	local zonedata = {
		polygon = polygon,
		draw = true,
		drawColor = newdata.color
	}
	game.interface.setZone(id, zonedata)
end

function InspectWindow:clearMaker(id)
	if (self._zonemarkers == nil) then
		return
	end
	self._zonemarkers[id] = nil
	game.interface.setZone(id)
end

function InspectWindow:clearMakers()
	if (self._zonemarkers == nil) then
		return
	end
	for id, v in pairs(self._zonemarkers) do
		game.interface.setZone(id)
	end
	self._zonemarkers = nil
end

function InspectWindow:onSelectEntityMouseEvent_cleanup()
	self:clearMakers()
end

local function hermitePos(t, pA, tA, pB, tB)
	local h00 = (2*t^3 - 3*t^2 + 1)
	local h10 = (t^3 - 2*t^2 + t)
	local h01 = (-2*t^3 + 3*t^2)
	local h11 = (t^3 - t^2)

	local pos1 = h00 * pA[1] + h10 * tA[1] + h01 * pB[1] + h11 * tB[1]
	local pos2 = h00 * pA[2] + h10 * tA[2] + h01 * pB[2] + h11 * tB[2]
	local pos3 = h00 * pA[3] + h10 * tA[3] + h01 * pB[3] + h11 * tB[3]
	return {
		pos1, pos2, pos3
	}
end

local function hermiteTangent(t, pA, tA, pB, tB)
	local ht00= (6*t^2 - 6*t)
	local ht10= (3*t^2 - 4*t + 1)
	local ht01= (-6*t^2 + 6*t)
	local ht11 = (3*t^2 - 2*t)


	local pos1 = ht00 * pA[1] + ht10 * tA[1] + ht01 * pB[1] + ht11 * tB[1]
	local pos2 = ht00 * pA[2] + ht10 * tA[2] + ht01 * pB[2] + ht11 * tB[2]
	local pos3 = ht00 * pA[3] + ht10 * tA[3] + ht01 * pB[3] + ht11 * tB[3]
	return {
		pos1, pos2, pos3
	}
end

local function getShapeForThing(entityid)
	--local bbox = api.engine.getComponent(entityid, api.type.ComponentType.BOUNDING_VOLUME)
	local compBaseNode = api.engine.getComponent(entityid, api.type.ComponentType.BASE_NODE)
	if (compBaseNode ~= nil) then
		local pos = { compBaseNode.position.x, compBaseNode.position.y, compBaseNode.position.z }
		return pos, zoneutil.makeCircleZone(pos, 5, 5), { 1.0, 1.0, 0, 0.7}
	end
	local compBaseEdge = api.engine.getComponent(entityid, api.type.ComponentType.BASE_EDGE)
	if (compBaseEdge ~= nil) then
		local compBaseEdgeStreet = api.engine.getComponent(entityid, api.type.ComponentType.BASE_EDGE_STREET)
		local compBaseEdgeTrack = api.engine.getComponent(entityid, api.type.ComponentType.BASE_EDGE_TRACK)

		local compNode0 = api.engine.getComponent(compBaseEdge.node0, api.type.ComponentType.BASE_NODE)
		local compNode1 = api.engine.getComponent(compBaseEdge.node1, api.type.ComponentType.BASE_NODE)

		local compNode0 = api.engine.getComponent(compBaseEdge.node0, api.type.ComponentType.BASE_NODE)
		local compNode1 = api.engine.getComponent(compBaseEdge.node1, api.type.ComponentType.BASE_NODE)

		local pos0 = vec3.toArray(compNode0.position)
		local pos1 = vec3.toArray(compNode1.position)
		local t0 = vec3.toArray(compBaseEdge.tangent0)
		local t1 = vec3.toArray(compBaseEdge.tangent1)

		--streetType = api.res.streetTypeRep.find(streetType)
		--end
		--local streetTypeDetail = api.res.streetTypeRep.get(streetType)
		--return streetTypeDetail.streetWidth + 2*streetTypeDetail.sidewalkWidth
		local edgeHalfWidth = 4

		if (compBaseEdgeStreet ~= nil) then
			local streetType = api.res.streetTypeRep.get(compBaseEdgeStreet.streetType)
			edgeHalfWidth = streetType.streetWidth/2+streetType.sidewalkWidth
		elseif(compBaseEdgeTrack ~= nil) then
			local trackType = api.res.trackTypeRep.get(compBaseEdgeTrack.trackType)
			edgeHalfWidth = trackType.shapeWidth/2
		end
		local points = {}
		local count = 8
		local rad = math.rad(90)
		for i = 0, count do
			local s = i/count
			local point = hermitePos(s, pos0, t0, pos1, t1)
			local t = hermiteTangent(s, pos0, t0, pos1, t1)
			local trotate = svec3.normalize(svec3.rotateZRad(t, rad))
			local displacement = svec3.mul(edgeHalfWidth, trotate)
			points[1+i] = svec3.add(point, displacement)
			points[1+(count*2)+1-i] = svec3.sub(point, displacement)
		end

		local pos = hermitePos(0.5, pos0, t0, pos1, t1)
		--[[
		for i, point in ipairs(points) do
			print(i," ", point[1], " ", point[2] )
		end
		--]]
		--commonapi.dmp(points)
		return pos, points

	end

	local compBV = api.engine.getComponent(entityid, api.type.ComponentType.BOUNDING_VOLUME)
	if (compBV and compBV.bbox ~= nil) then
		local bbox = compBV.bbox
		local points = {
			{ bbox.min.x, bbox.min.y },
			{ bbox.min.x, bbox.max.y },
			{ bbox.max.x, bbox.max.y },
			{ bbox.max.x, bbox.min.y }
		}
		local pos = {
			(bbox.min.x + (0.5* (bbox.max.x - bbox.min.x))),
			(bbox.min.y + (0.5* (bbox.max.y - bbox.min.y))),
			(bbox.min.z + (0.5* (bbox.max.z - bbox.min.z))),
		}
		return pos, points
	end

	local compParcel = api.engine.getComponent(entityid, api.type.ComponentType.PARCEL)
	if (hasParcelFace == true and compParcel ~= nil) then
		if (compParcel.faces[1] ~= nil) then
			local points = {}
			for i, face in ipairs(compParcel.faces) do
				for fi, point in ipairs(face.points) do
					points[#points+1] = { point.x,  point.y,  point.z }
				end
			end
			if (#points > 0) then
				return points[1], points
			end
		end
	end
	return nil
end

local function getBBoxCenter(entityid)
	local compBV = api.engine.getComponent(entityid, api.type.ComponentType.BOUNDING_VOLUME)
	if (compBV and compBV.bbox ~= nil) then
		local bbox = compBV.bbox
		local dx = (bbox.max.x - bbox.min.x) / 2
		local dy = (bbox.max.y - bbox.min.y) / 2
		local dz = (bbox.max.z - bbox.min.z) / 2
		local epos = {
			bbox.min.x + dx,
			bbox.min.y + dy,
			bbox.min.z + dz
		}

		return epos
		--commonapi.dmp(bbox)
		--commonapi.dmp(epos)
	end
end


function InspectWindow:onShowEntityOnMap(entityId)

	if (entityId < 1) then
		self:clearMaker("selectcursor")
		return
	end

	local pos, polygon, color = getShapeForThing(entityId)

	if (pos) then
		if (color == nil) then
			color = {.0, .0, 1.0, .7}
		end
		self:zoneMakerChange("selectcursor", pos, 5, color, polygon)

		local controller = api.gui.util.getGameUI():getMainRendererComponent():getCameraController()
		--controller:focus(tonumber(win.params.EntityId), true)

		local campos, camrot, campitch = controller:getCameraData()
		local camposNew = api.type.Vec2f.new()
		camposNew.x = pos[1]
		camposNew.y = pos[2]
		controller:setCameraData(camposNew, 100, camrot, campitch)
		return
	end


	self:clearMaker("selectcursor")
end


function InspectWindow:onSelectEntityMouseEvent(evt, mouseScreenPos)
	if (self._ugwindow == nil) then
		secondaryui.setMouseListener(nil)
		return
	end
	if (evt.type == 2 and evt.button == 2) then
		secondaryui.setMouseListener(nil)
		return
	end
	local _ugwindow = self._ugwindow
	local rect = _ugwindow:getContentRect()
	if (rect:contains(mouseScreenPos.x, mouseScreenPos.y)) then
		return
	end
	local pos = game.gui.getTerrainPos()
	if (pos == nil) then
		return
	end


	local entities = game.interface.getEntities({
			radius = 10,
			pos = {
				pos[1],
				pos[2],
				pos[3]
			}
		}, { includeData = true }
	)

	local entityDist = {}

	local addEntityWithPos = function(entityid, epos)
		if (epos ~= nil) then
			local dx = epos[1]-pos[1]
			local dy = epos[2]-pos[2]
			local dz = (epos[3] or 0)-pos[3]
			local dist2 = dx*dx + dy*dy + dz*dz
			entityDist[#entityDist+1] = {
				id = entityid,
				pos = epos,
				d = dist2
			}
		end
	end

	for entityid, entitydata in pairs(entities) do
		local epos
		if (entityid > 0) then
			if ( entitydata["type"] == "VEHICLE" and entitydata.carrier == "RAIL") then
				local compTrain = api.engine.getComponent(entityid, api.type.ComponentType.TRAIN)
				for vehicleidx, vehicleentityid in pairs(compTrain.vehicles) do
					local bboxcenter = getBBoxCenter(vehicleentityid)
					addEntityWithPos(entityid, bboxcenter)
				end
			end

			if (entitydata.position ~= nil) then
				epos = entitydata.position
			elseif (entitydata.node0pos ~= nil) then
				epos = hermitePos(0.5, entitydata.node0pos, entitydata.node0tangent, entitydata.node1pos, entitydata.node1tangent)
				--self:zoneMakerChange("loc"..entityid, epos, 4)
			else
				epos = getBBoxCenter(entityid)
			end
			addEntityWithPos(entityid, epos)
		end

	end

	if (#entityDist > 0) then
		table.sort(entityDist,
			function(a,b)
				return (a.d < b.d)
			end
		)
		local xpos, polygon, color = getShapeForThing(entityDist[1].id, entityDist[1].pos)
		if (color == nil) then
			color = {.0, .0, 1.0, .7}
		end
		self:zoneMakerChange("selectcursor", entityDist[1].pos, 5, color, polygon)
	else
		self:clearMaker("selectcursor")
	end

	if (evt.type == 2 and evt.button == 0) then
		--print("evt.type == 2 and evt.button == 1")
		if (#entityDist > 0) then
			self.params.EntityId = entityDist[1].id
		end

		secondaryui.setMouseListener(nil)
		self:refresh()
		return true
	end

	self:zoneMakerChange("inspectcursor", pos, 10)

	if (evt.type < 3 and evt.button == 0) then
		return true;
	end
end

local typid_ShapeList = api.type.ComponentType.SHAPE_LIST
local addonData = {

}

local function inspectShapeListAttributes(entityId, componentData)
	local lines = {"\n"}
	if (commonapi._native and commonapi._native.StreetTypes) then
		for idx, shape in ipairs(componentData.shapes) do
			lines[#lines+1] = "Shape "..idx.."\n"
			local attributes = commonapi._native.StreetTypes.getShapeAttributes(shape)
			local attributesData = attributes.data
			for key, val in pairs(attributesData) do
				local ok, result = pcall(inspectlua, val, { level = 4 })
				lines[#lines+1] = "  Key "..key..": " .. result
			end
		end
	end
	return table.concat(lines, "\n")
end


local function getComponentData(entityId, componentName)
	if (componentName == "SHAPE_LIST(_ATTRIBUTES)") then
		componentName = "SHAPE_LIST"
	end
	local componentTypeId = api.type.ComponentType[componentName]
	local componentData = api.engine.getComponent(entityId, componentTypeId)
	return componentData
end

local function convertComponentData2Str(entityId, componentName, componentData)
	if (componentName == "SHAPE_LIST(_ATTRIBUTES)") then
		if (commonapi._native and commonapi._native.StreetTypes) then
			return inspectShapeListAttributes(entityId, componentData)
		else
			return "CommonAPI2 native missing"
		end
	end
	local str = inspectlua(componentData)
	return str
end

local function inspectComponent(entityId, componentTypeId, componentData)
	if (componentData == nil) then
		componentData = api.engine.getComponent(entityId, componentTypeId)
	end
	local str = inspectlua(componentData)
	if (componentTypeId == typid_ShapeList) then
		local data = inspectShapeListAttributes(entityId, componentTypeId, componentData)
		str = str .. data
	end
	return str
end

function InspectWindow:createUIListTabEntity()
	local uilist = {}

	local entityId = tonumber(self.params.EntityId)

	local uilistbuttonsInput = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttonsInput
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "TextInput",
		id = "EntityId",
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		id = "ShowEntityData",
		caption = "Show",
		onclick = function(item, win)
			win:refresh()
		end
	}

	local stateTexts = {
		self._("Select entity by mouse"),
		self._("Stop selecting entity")
	}

	local curState = stateTexts[1]
	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		id = "EntityStartMouseSelector",
		caption = curState,
		onclick = function(item, win)
			local buttoneleid = win._ugidprefix .. "EntityStartMouseSelector"
			local button = api.gui.util.getById(buttoneleid)
			local buttonText = button:getLayout():getItem(0)
			if (secondaryui.hasMouseListener()) then
				secondaryui.setMouseListener(nil)
				buttonText:setText(stateTexts[1])
			else
				secondaryui.setMouseListener({
					filterui = false,
					fn = function(evt, mouseScreenPos)
						return win:onSelectEntityMouseEvent(evt, mouseScreenPos)
					end,
					cleanup = function()
						win:onSelectEntityMouseEvent_cleanup()
					end,
				})
				buttonText:setText(stateTexts[2])
			end
		end
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		caption = self._("Last selected"),
		id = "EntityGetCurrrent",
		onclick = function(item, win)
			self.params.EntityId = secondaryui.lastEntitySelected
			win:refresh()
		end
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		--caption = self._("Last selected"),
		image = "ui/eye19.tga",
		id = "EntityShowOnMap",
		onclick = function(item, win)
			local controller = api.gui.util.getGameUI():getMainRendererComponent():getCameraController()
			controller:focus(tonumber(win.params.EntityId), true)
		end
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		--caption = self._("Last selected"),
		image = "ui/eye19.tga",
		id = "EntityShowOnMap2",
		onclick = function(item, win)
			win:onShowEntityOnMap(tonumber(win.params.EntityId))
		end
	}


	if (self.params.previousEntityId ~= nil) then
		uilistbuttonsInput[#uilistbuttonsInput + 1] = {
			typ = "Button",
			caption = self._("Previous Entity: ")..tostring(self.params.previousEntityId),
			id = "EntityGetPrevious",
			onclick = function(item, win)
				local previous = self.params.EntityId
				self.params.EntityId = self.params.previousEntityId
				self.params.previousEntityId = previous
				win:refresh()
			end
		}
	end

	local componentTypes = getmetatable(api.type.ComponentType).__index
	local componentTypesKeys = {}

	for k in pairs(componentTypes) do
		componentTypesKeys[#componentTypesKeys + 1] = k
	end
	componentTypesKeys[#componentTypesKeys + 1] = "SHAPE_LIST(_ATTRIBUTES)"
	table.sort(componentTypesKeys)

	local uilistregion = {}

	if (entityId ~= nil and entityId >= 0) then
		uilist[#uilist + 1] = {
			typ = "Label",
			caption = "Entity ID: " .. entityId
		}

		uilist[#uilist + 1] = {
			typ = "ScrollRegion",
			id = "ScrollRegion",
			border = false,
			uilist = uilistregion
		}
		local filterCName = self.params.EntityFilter or ""
		local filterList = {}

		local filtermap = {
			EntityFilterAccount = "ACCOUNT",
			EntityFilterLotList = "LOT_LIST",
			EntityFilterLogBook = "LOG_BOOK",
			EntityFilterModelInstanceList = "MODEL_INSTANCE_LIST"
		}

		for pkey, comtype in pairs(filtermap) do
			if (self.params[pkey] == true) then
				filterList[comtype] = true
			else
				filterList[comtype] = nil
			end
		end

		if (api.engine.entityExists(entityId)) then
			local fnEntry = function(entityId, componentName)
				local componentTypeId = api.type.ComponentType[componentName]
				local componentData = getComponentData(entityId, componentName)

				if (componentData ~= nil) then
					uilistregion[#uilistregion + 1] = {
						typ = "Separator"
					}

					local uilistregionHeaderLayout = {}
					uilistregion[#uilistregion + 1] = {
						typ = "LayoutHorizontal",
						uilist = uilistregionHeaderLayout
					}

					local componentDataAsString = ""
					if (self.params["show_"..componentName] == nil) then
						self.params["show_"..componentName] = true

						componentDataAsString = convertComponentData2Str(entityId, componentName, componentData)
						if ( componentDataAsString:len() > 1000) then
							componentDataAsString = ""
							self.params["show_"..componentName] = false
						end
					else
						if (self.params["show_"..componentName] == true) then
							componentDataAsString = convertComponentData2Str(entityId, componentName, componentData)
						end
					end

					uilistregionHeaderLayout[#uilistregionHeaderLayout + 1] = {
						typ = "Checkbox",
						id = "show_"..componentName,
						onclick = function(item, win)
							win:refreshKeepPos()
						end
					}

					uilistregionHeaderLayout[#uilistregionHeaderLayout + 1] = {
						typ = "Label",
						caption = componentName
					}

					if (getRefsFn[componentName] ~= nil) then
						local refs = getRefsFn[componentName](entityId, componentData)
						local refCnt = 1
						for refsGroup, refData in pairs(refs) do
							if (#refData > 0) then
								local uirefs = {}
								uilistregion[#uilistregion + 1] = {
									typ = "LayoutHorizontal",
									uilist = uirefs
								}
								uirefs[#uirefs + 1] = {
									typ = "Label",
									caption = refsGroup,
								}
								for refNum, refId in ipairs(refData) do
									uirefs[#uirefs + 1] = {
										typ = "Button",
										caption = ""..refId,
										id = "dataRef_"..componentName.."_"..refCnt.. "_"..refNum,
										onclick = function(item, win)
											self.params.EntityId = refId
											self.params.previousEntityId = entityId
											win:refresh()
										end
									}
									uirefs[#uirefs + 1] = {
										typ = "Button",
										image = "ui/eye19.tga",
										id = "dataRefShow_"..componentName.."_"..refCnt.. "_"..refNum,
										onclick = function(item, win)
											win:onShowEntityOnMap(tonumber(refId))
										end
									}
								end
							end
							refCnt = refCnt + 1
						end
					end

					if (componentDataAsString ~= "") then
						uilistregion[#uilistregion + 1] = {
							typ = "Label",
							caption = componentDataAsString,
							id = "data_" .. componentName
						}
					end

					uilistregionHeaderLayout[#uilistregionHeaderLayout + 1] = {
						typ = "Button",
						id = "datatoconsole_" .. componentName,
						--caption = self._("to Console"),
						image = "ui/manaul_installed_mod_icon.tga",
						gravity = { -1, 0.5 },
						onclick = function(item, win)
							win:showComponentInConsole(entityId, componentName)
						end
					}

					if (commonapi._native ~= nil and commonapi._native.utils ~= nil) then
						uilistregionHeaderLayout[#uilistregionHeaderLayout + 1] = {
							typ = "Button",
							id = "datatoclipboard_" .. componentName,
							--caption = self._("to Console"),
							--image = "ui/manaul_installed_mod_icon.tga",
							caption = self._("Copy"),
							onclick = function(item, win)
								local compData = getComponentData(entityId, componentName)
								local str = convertComponentData2Str(entityId, componentName, compData)
								commonapi._native.utils.setClipboardText(str)
							end
						}
					end
				end
			end

			if (filterCName ~= "" and api.type.ComponentType[filterCName] ~= nil) then
				fnEntry(entityId, filterCName)
			else
				for _, componentName in ipairs(componentTypesKeys) do
					fnEntry(entityId, componentName)
				end
			end
		end
	else
		uilist[#uilist + 1] = {
			typ = "Label",
			caption = "Entity ID: <not found>"
		}
	end

	uilist[#uilist + 1] = {
		typ = "Separator"
	}

	local uilistbuttons = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttons
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "Refresh",
		caption = self._("Refresh"),
		onclick = function(item, win)
			win:refresh()
		end
	}


	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "CloseAll",
		caption = self._("Close all"),
		onclick = function(item, win)
			for _, componentName in ipairs(componentTypesKeys) do
				self.params["show_"..componentName] = false
			end
			win:refreshKeepPos()
		end
	}
	
	--65359

	--self.uilist = uilist
	return uilist
end

function InspectWindow:createUIListTabEvents()
	local uilist = {}
	local uilistregion = {}
	local uilistbuttons = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttons
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "EventTraceStart",
		caption = self._("Start"),
		onclick = function(item, win)
			local traceGameEvents = (self.params["EventTraceGameEvents"] == true)
			EventTraceOutputStart(traceGameEvents)
		end
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "EventTraceStop",
		caption = self._("Stop"),
		onclick = function(item, win)
			EventTraceOutputStop()
		end
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "ClearEventTrace",
		caption = self._("Clear"),
		onclick = function(item, win)
			local eleid = win._ugidprefix .. "EventTraceScrollRegion"
			local ele = api.gui.util.getById(eleid)
			if (ele ~= nil) then
				local layout = ele:getContent():getLayout()
				local numitems = layout:getNumItems()
				if (numitems > 0) then
					for i = numitems, 1, -1 do
						local tmp = layout:getItem(i - 1)
						layout:removeItem(tmp)
						tmp:destroy()
					end
				end
			end
		end
	}
	if (commonapi._native ~= nil and commonapi._native.utils ~= nil) then
		uilistbuttons[#uilistbuttons + 1] = {
			typ = "Button",
			id = "CopyEventTrace",
			caption = self._("Copy"),
			onclick = function(item, win)
				local eleid = win._ugidprefix .. "EventTraceScrollRegion"
				local ele = api.gui.util.getById(eleid)
				if (ele ~= nil) then
					local layout = ele:getContent():getLayout()
					local numitems = layout:getNumItems()
					if (numitems > 0) then
						local text = "";
						for i = 0, numitems-1 do
							local eletext = layout:getItem(i)
							if (eletext.getText ~= nil) then
								text = text .. eletext:getText()
							end
						end
						commonapi._native.utils.setClipboardText(text)
					end
				end
			end
		}
	end
	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Label",
		caption = " Trace game events via stdout.txt",
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Checkbox",
		id = "EventTraceGameEvents",
		default = false,
		onclick = function(item, win)

		end
	}

	uilist[#uilist + 1] = {
		typ = "ScrollRegion",
		id = "EventTraceScrollRegion",
		border = true,
		uilist = uilistregion
	}

	uilist[#uilist + 1] = {
		typ = "Separator"
	}

	return uilist
end

function InspectWindow:setNewElement(newele)
	if (newele == nil) then
		self.params.componentId = nil
		self.params.componentRef = nil
		self.params.componentRefChecker = nil
		return true
	end
	if (newele.getId ~= nil) then
		local id = newele:getId()
		if (id ~= nil and id ~= "") then
			print("setNewElement: using id", id)
			self.params.componentId = id
			self.params.componentRef = nil
			self.params.componentRefChecker = nil
			return true
		end
	end
	if (newele.addLifeTimeChecker ~= nil) then
		print("setNewElement: using addLifeTimeChecker")
		self.params.componentId = ""
		self.params.componentRefChecker = newele:addLifeTimeChecker()
		self.params.componentRef = newele
		return true
	else
		print("setNewElement: using directly")
		self.params.componentId = ""
		self.params.componentRefChecker = nil
		self.params.componentRef = newele
		return true
	end
	return false
end

function InspectWindow:getCurrentEle()
	local ele
	if (self.params.componentId ~= nil and self.params.componentId ~= "") then
		ele = api.gui.util.getById( self.params.componentId )
		if (ele) then
			self.params.componentRef = nil
			self.params.componentRefChecker = nil
		end
	elseif (self.params.componentRefChecker ~= nil) then
		if (api.gui.util.check(self.params.componentRefChecker)) then
			ele = self.params.componentRef
		end
	elseif (self.params.componentRef ~= nil) then
		ele = self.params.componentRef
	end
	return ele
end

local getRefInfo = function(ele)
	local infos = {}
	infos[#infos+1] = (ui._getComponentType(ele) or "UnknownComponent").."\n"
	if (ele.getName ~= nil) then
		infos[#infos+1] = "name: "..(ele:getName() or "")
	end
	if (ele.getId ~= nil) then
		infos[#infos+1] = "id: "..(ele:getId() or "")
	end
	return table.concat(infos, " ")
end

local findWindowElementFilter = {
	ToolTipContainer = true,
	mainMenuTopBar = true
}
local function findWindowElement(layout, mouse, deep, maxdeep)
	if (layout.getNumItems == nil) then
		return nil
	end
	if (deep > maxdeep) then
		return
	end
	local numItems = layout:getNumItems()
	for i = 0, numItems-1 do
		local item = layout:getItem(i)
		if (item.getName ~= nil) then
			local name = item:getName()
			if (findWindowElementFilter[name] == true) then
				goto continueItems
			end

		end
		if (item.isVisible ~= nil) then
			if item:isVisible() and item:getContentRect():contains(mouse.x, mouse.y) then
				if (item.getLayout ~= nil) then
					local itemlayout = item:getLayout()
					if (itemlayout ~= nil) then
						local child = findWindowElement(itemlayout, mouse, deep+1, maxdeep)
						if (child ~= nil) then
							--print("Found child at deep", deep)
							--print("Found child at deep", deep, getRefInfo(child))
							return child
						end
					end
				end
				return item
			end
		elseif(item.getLayout == nil and item.getNumItems ~= nil and item.getItem ~= nil) then
			-- Could be something like a CBoxLayout
			local child = findWindowElement(item, mouse, deep+1, maxdeep)
			if (child ~= nil) then
				--print("Found child at deep", deep)
				--print("Found child at deep", deep, getRefInfo(child))
				return child
			end
		end
::continueItems::
	end
	return nil
end

function InspectWindow:onSelectUIElementMouseEvent_cleanup()
	local x = self.__curelementmouse
	if (x ~= nil) then
		--x:setHighlighted(false)
		if (x.removeStyleClass ~= nil) then
			x:removeStyleClass("commonapi-cathighlight")
		end
	end
	self.__curelementmouse = nil
end

function InspectWindow:onSelectUIElementMouseEvent(evt, mouseScreenPos)
	if (self._ugwindow == nil) then
		secondaryui.setMouseListener(nil)
		return
	end
	local _ugwindow = self._ugwindow
	local rect = _ugwindow:getContentRect()
	if (rect:contains(mouseScreenPos.x, mouseScreenPos.y)) then
		--print("In own window")
	else
		if (evt.type == 2 and evt.button == 1) then
			--print("evt.type == 2 and evt.button == 1")
			secondaryui.setMouseListener(nil)
			return
		end
		local mainView = api.gui.util.getById("mainView")
		local mainLayout = mainView:getLayout()
		local layer2 = mainLayout:getItem(2)
		local viewManagerLayout = layer2:getLayout():getItem(0):getLayout():getItem(0):getLayout()
		local uiRootLayout = layer2:getLayout():getItem(1):getLayout()
		local element = findWindowElement(viewManagerLayout, mouseScreenPos, 0, 100)
		if (element == nil) then
			element = findWindowElement(uiRootLayout, mouseScreenPos, 0, 100)
		end


		local olele = self.__curelementmouse
		if (olele ~= nil) then
			if (olele ~= element) then
				--self.__curelementmouse:setHighlighted(false)
				if (olele.removeStyleClass ~= nil) then
					olele:removeStyleClass("commonapi-cathighlight")
				end
				self.__curelementmouse = nil
			end
		end


		if (element ~= nil and olele ~= element) then
			local childinfo = getRefInfo(element) or "UnknownComponent"
		--	print("Found new child", childinfo)
		end
		if (element ~= nil) then
			if (olele ~= element) then
				self.__curelementmouse = element
				--element:setHighlighted(true)
				if (element.addStyleClass ~= nil) then
					element:addStyleClass("commonapi-cathighlight")
				end
			end
		end

		if (evt.type < 3 and evt.button == 2) then
			--print("evt.type == 2 and evt.button == 0")
			secondaryui.setMouseListener(nil)
			self:setNewElement(element)
			self:refresh()
			return true
		end
	end
end


function InspectWindow:createUIListTabUI()
	local uilist = {}

	local uilistbuttonsInput = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttonsInput
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "TextInput",
		id = "componentId",
		default = "mainView",
	}

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		id = "UIRefresh",
		caption = "Refresh",
		onclick = function(item, win)
			win:refresh()
		end
	}


	local stateTexts = {
		"Select by Mouse",
		"Click for end or right click on ui element"
	}
	local curState = stateTexts[1]
	if (secondaryui.hasMouseListener()) then
		curState = stateTexts[2]
	end

	uilistbuttonsInput[#uilistbuttonsInput + 1] = {
		typ = "Button",
		id = "UISelectElement",
		caption = curState,
		onclick = function(item, win)
			local buttoneleid = win._ugidprefix .. "UISelectElement"
			local button = api.gui.util.getById(buttoneleid)
			local buttonText = button:getLayout():getItem(0)
			if (secondaryui.hasMouseListener()) then
				secondaryui.setMouseListener(nil)
				buttonText:setText(stateTexts[1])
			else
				secondaryui.setMouseListener({
					filterui = false,
					fn = function(data, mouseScreenPos)
						return win:onSelectUIElementMouseEvent(data, mouseScreenPos)
					end,
					cleanup = function()
						win:onSelectUIElementMouseEvent_cleanup()
					end,
				})
				buttonText:setText(stateTexts[2])
			end
		end
	}


	local ele = self:getCurrentEle()
	if (ele ~= nil) then
		local uiTableData = {}
		uilist[#uilist + 1] = {
			typ = "Table",
			id = "UIComponentDataTable",
			columns = 2,
			border = false,
			uilist = uiTableData,
			size = {
				0,
				-100
			},
			flags = {

			},
		}
		local function createItem(caption, data)
			uiTableData[#uiTableData + 1] = {
				typ = "Label",
				caption = caption,
				column = 0,
				rownew = true,
			}
			if (type(data) == "string") then
				uiTableData[#uiTableData + 1] = {
					typ = "Label",
					caption = data,
					column = 1
				}
			else
				data.column = 1
				uiTableData[#uiTableData + 1] = data
			end
		end


		local function createSubItem(t, caption, data)
			t[#t + 1] = {
				typ = "Label",
				caption = caption,
				column = 0,
				rownew = true,
			}
			if (type(data) == "string") then
				t[#t + 1] = {
					typ = "Label",
					caption = data,
					column = 1
				}
			else
				data.column = 1
				t[#t + 1] = data
			end
		end

		local componentType = ui._getComponentType(ele)
		createItem("Component Typ", componentType or "unknown")




		if (ele.getName ~= nil) then
			local name = ele:getName()
			if (name ~= nil) then
				createItem("Name", name)
			end
		end

		if (ele.getId ~= nil) then
			local currentId = ele:getId()
			if (currentId ~= nil and currentId ~= "") then
				local textgetById = " api.gui.util.getById(\""..currentId.."\")"
				createItem("Id", {
					typ = "LayoutHorizontal",
					uilist = {
						{
							typ = "Label",
							caption = currentId,
						},
						{
							typ = "Button",
							id = "UIComponentIdToConsole",
							--caption = self._("to Console"),
							image = "ui/manaul_installed_mod_icon.tga",
							gravity = { -1, 0.5 },
							onclick = function(item, win)
								win:addTextToConsole(textgetById)
							end
						}
					}
				})
			else
				createItem("GenerateId", {
					typ = "LayoutHorizontal",
					uilist = {
						{
							typ = "Label",
							caption = "Generate a temporary id",
						},
						{
							typ = "Button",
							id = "UIComponentIdGenerate",
							image = "ui/b_arrow_right.tga",
							gravity = { -1, 0.5 },
							onclick = function(item, win)
								local curele = win:getCurrentEle()
								local newid = tostring(curele):match("[A-Za-f0-9]+$")
								curele:setId("temp"..newid)
								win:refresh()
							end
						}
					}
				})
			end
		end

		if (ele.getContentRect ~= nil) then
			local rect = ele:getContentRect()
			createItem("Rect: ", "(x: ${x}, y: ${y}, w: ${w}, h: h${h})" % rect)
		end

		if ( commonapi._native ~= nil) then
			if (componentType == "sol.UI::ImageView*") then
				local imagename = commonapi._native.UIHacks.getImageViewImage(ele);
				if (imagename ~= nil) then
					createItem("Image:", imagename)
				end
			end
			local styles = commonapi._native.UIHacks.getStyleClasses(ele);
			if (styles ~= nil and #styles > 0) then
				createItem("Styleclasses:", ""..table.concat(styles,", "))
			end
		end

		if (ele.getParent ~= nil) then
			local parent = ele:getParent()
			if (parent ~= nil) then
				local infoButtonOrText = "<none>"
				local info = getRefInfo(parent)
				if (info ~= nil) then
					infoButtonOrText = {
						typ = "Button",
						caption = info,
						id = "UILayoutParent",
						onclick = function(item, win)
							local curele = win:getCurrentEle()
							local newele = curele:getParent()
							if (win:setNewElement(newele)) then
								win:refresh()
							end
						end
					}
				end
				createItem("Element Parent", infoButtonOrText)
			end
		end

		if (ele.getComponent ~= nil) then
			local parent = ele:getComponent()
			if (parent ~= nil) then
				local infoButtonOrText = "<none>"
				local info = getRefInfo(parent)
				if (info ~= nil) then
					infoButtonOrText = {
						typ = "Button",
						caption = info,
						id = "UIComponentGoToParent",
						onclick = function(item, win)
							local curele = win:getCurrentEle()
							local newele = curele:getComponent()
							if (win:setNewElement(newele)) then
								win:refresh()
							end
						end
					}
				end
				createItem("getComponent", infoButtonOrText)
			end
		end

		local createChildrenList = function(baseid, caption, itemele, itemeletype)
				local uilayoutitems = {}

				createItem(caption,
					{
						typ = "Table",
						id = baseid.."children",
						columns = 2,
						border = true,
						uilist = uilayoutitems,
						--size = {
						--	0,
						--	100
						--},
						flags = {

						},
					}
				);
				if (itemeletype == 0) then
					createSubItem(uilayoutitems, "Typ", (ui._getComponentType(itemele) or " "))
				end
				local numitems = -1
				if (itemele.getNumItems ~= nil) then
					numitems = itemele:getNumItems()
				end
				createSubItem(uilayoutitems, "NumItems", tostring(numitems))
				if (itemele.getItem ~= nil) then
					if (numitems >= 0) then
						for i = 0, numitems-1 do
							local item = itemele:getItem(i)
							createSubItem(uilayoutitems, tostring(i), {
								typ = "Button",
								caption = getRefInfo(item),
								id =  baseid.."Child"..i,
								onclick = function(item, win)
									local curele = win:getCurrentEle()
									local newele
									if (itemeletype == 1) then
										newele = curele:getItem(i)
									else
										newele = curele:getLayout():getItem(i)
									end
									if (win:setNewElement(newele)) then
										win:refresh()
									end
								end
							})
						end
					end
				end 
		end -- function


		if (ele.getLayout ~= nil) then
			local layout = ele:getLayout()
			if (layout ~= nil) then
				createChildrenList("UILayout", "Layout:", layout, 0)
			else
				createItem("Layout:", "<nil>")
			end
		end


		if (ele.getItem ~= nil and ele.getNumItems ~= nil)  then
			createChildrenList("UIChildren", "ElementChildren:", ele, 1)
		end

	else
		uilist[#uilist + 1] = {
			typ = "Label",
			caption = "Component not found or invalid"
		}
	end

	uilist[#uilist + 1] = {
		typ = "Separator"
	}

	local uilistbuttons = {}
	uilist[#uilist + 1] = {
		typ = "LayoutHorizontal",
		uilist = uilistbuttons
	}

	uilistbuttons[#uilistbuttons + 1] = {
		typ = "Button",
		id = "UIRefreshComponent2",
		caption = self._("Refresh"),
		onclick = function(item, win)
			win:refresh()
		end
	}
	return uilist
end



return InspectWindow
